import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.employment_details import EmploymentDetails
from ..models.item import Item
from ..models.national_insurance_calculation import NationalInsuranceCalculation
from ..models.pay_options import PayOptions
from ..models.pay_periods import PayPeriods
from ..models.pay_run_state import PayRunState
from ..models.pay_run_totals import PayRunTotals
from ..models.pension_summary import PensionSummary
from ..models.personal_details import PersonalDetails
from ..models.tax_year import TaxYear
from ..models.umbrella_payment import UmbrellaPayment
from ..models.value_override import ValueOverride
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRunEntry")


@attr.s(auto_attribs=True)
class PayRunEntry:
    """A PayRun contains multiple PayRunEntries, one for each Employee that is being paid.
    The value for <code>PersonalDetails</code> and <code>EmploymentDetails</code> is copied from the Employee record.
    So to change them you should update the Employee, not the PayRunEntry.
    Changes made to the Employee are only reflected whilst the PayRun is still open.
    Once you finalise and close the PayRun then a snapshot is taken of these values which is stored with the PayRunEntry
    for future reference.

        Attributes:
            tax_year (Union[Unset, TaxYear]):
            tax_month (Union[Unset, int]): [readonly] The Tax Month that the Payment Date falls in
            start_date (Union[Unset, datetime.date]): [readonly]
            end_date (Union[Unset, datetime.date]): [readonly]
            note (Union[Unset, None, str]): Any note that you'd like to appear on the payslip
            bacs_sub_reference (Union[Unset, None, str]): [readonly] A randomly generated string for use with the RTI Hash
                Cross Reference
            bacs_hash_code (Union[Unset, None, str]): [readonly] A Hash Code used for RTI BACS Hash Cross Reference
            percentage_of_working_days_paid_as_normal (Union[Unset, float]): [readonly] If the employee is paid a set amount
                per period (ie, not an hourly or daily rate) and there is any Leave that is either Not Paid or Statutory Pay
                then this value
                will give the percentage of working days (based on the Working Pattern) that should be paid as normal.
                If there is no Leave in the period or PayOptions.AutoAdjustForLeave is false, then this will be 1.
            working_days_not_paid_as_normal (Union[Unset, float]): [readonly] If PercentageOfWorkingDaysPaidAsNormal has a
                value other than 1
                then this property will tell you how many working days have been deducted from the basic pay
                due to either being Not Paid or Statutory Pay
            pay_period (Union[Unset, PayPeriods]):
            ordinal (Union[Unset, int]): [readonly] Indicates whether this uses the first, second, third (etc) PaySchedule
                for this PayPeriod.
            period (Union[Unset, int]): [readonly] The Tax Week or Tax Month number this PayRunEntry relates to
            is_new_starter (Union[Unset, bool]): Determines whether or not this Employee will be declared as a new starter
                on the resulting FPS
            unpaid_absence (Union[Unset, bool]): [readonly] Indicates that there was unpaid absence in the pay period
            has_attachment_orders (Union[Unset, bool]): [readonly] Indicates that there are AttachmentOrders for this
                Employee in this entry
            payment_date (Union[Unset, datetime.date]): The date this payment was or will be made
            prior_payroll_code (Union[Unset, None, str]): [readonly] If the FPS needs to declare a change of PayId then this
                will contain the previous code
                It's worked out automatically but can also be set from the Employees EmploymentDetails property.
            pension_summary (Union[Unset, PensionSummary]): If a PayRunEntry contains pension contributions then it'll also
                include a PensionSummary model
                giving further information about the Pension Scheme and the contributions made
            employee (Union[Unset, Item]):
            totals (Union[Unset, PayRunTotals]): Used to represent totals for a PayRun or PayRunEntry.
                If a value is 0 then it will not be shown in the JSON.
            period_overrides (Union[Unset, None, List[ValueOverride]]): Any calculated values for this period that should be
                overridden with a different value
            totals_ytd (Union[Unset, PayRunTotals]): Used to represent totals for a PayRun or PayRunEntry.
                If a value is 0 then it will not be shown in the JSON.
            totals_ytd_overrides (Union[Unset, None, List[ValueOverride]]): Any values of TotalsYtd that should be
                overridden with a different value
            forced_cis_vat_amount (Union[Unset, None, float]): If this employee is a CIS Subcontractor registered for VAT,
                we'll automatically work out VAT at the set rate.
                If you want to override this calculations then set this property to anything other than null.
            holiday_accrued (Union[Unset, float]): The amount of holiday days accrued in the period.
            state (Union[Unset, PayRunState]): The state of the payrun. You would set this value when updating a payrun to
                finalise or re-open it.
                Other states are used with Bureau functionality which isn't currently generally available.
            is_closed (Union[Unset, bool]): [readonly] Set to True if the PayRun is Finalised and changes can no longer be
                made to the PayRunEntries
            manual_ni (Union[Unset, bool]): If set to true then you must provide your own value for
                NationalInsuranceCalculation.
                You'd normally leave this set to false and let us automatically calculate NI amounts.
            national_insurance_calculation (Union[Unset, NationalInsuranceCalculation]): Included as part of the PayRunEntry
                model to provide details of how the National Insurance Contribution was calculated.
                Unless the PayRunEntry.ManualNi property is set to true then these value will all be read-only and are
                recalculated everytime a payrun is updated.
                This calculation could be made up of one or more calculations made on different NI table letters.
                Where more than NI table letter affects the calculation, the calculation for each NI table letter will be
                contain in the Breakdown.
            payroll_code_changed (Union[Unset, bool]): [readonly] Indicates whether or not the Payroll Code for this
                Employee has changed since the last FPS
            ae_not_enroled_warning (Union[Unset, bool]): [readonly] If true then this Employee needs to be on an Auto
                Enrolment pension but isn't yet.
            fps (Union[Unset, Item]):
            email_id (Union[Unset, None, str]): If the Payslip for this PayRunEntry has been emailed to the employee then
                the Id for an EmployerEmail will be provided here.
                if the value is all zeroes then the email is in the process of being created.
            recieving_offset_pay (Union[Unset, bool]): If the pay is being topped up due to an applied Leave having the
                offset value set to true then
                this will be set to true
            payment_after_leaving (Union[Unset, bool]): [readonly] If this payment is for an employee that has left then
                this is set to true.
            umbrella_payment (Union[Unset, UmbrellaPayment]):
            id (Union[Unset, str]): [readonly] The unique id of the object
            personal_details (Union[Unset, PersonalDetails]):
            employment_details (Union[Unset, EmploymentDetails]):
            pay_options (Union[Unset, PayOptions]): This object forms the basis of the Employees payment.
    """

    tax_year: Union[Unset, TaxYear] = UNSET
    tax_month: Union[Unset, int] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    end_date: Union[Unset, datetime.date] = UNSET
    note: Union[Unset, None, str] = UNSET
    bacs_sub_reference: Union[Unset, None, str] = UNSET
    bacs_hash_code: Union[Unset, None, str] = UNSET
    percentage_of_working_days_paid_as_normal: Union[Unset, float] = UNSET
    working_days_not_paid_as_normal: Union[Unset, float] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    period: Union[Unset, int] = UNSET
    is_new_starter: Union[Unset, bool] = UNSET
    unpaid_absence: Union[Unset, bool] = UNSET
    has_attachment_orders: Union[Unset, bool] = UNSET
    payment_date: Union[Unset, datetime.date] = UNSET
    prior_payroll_code: Union[Unset, None, str] = UNSET
    pension_summary: Union[Unset, PensionSummary] = UNSET
    employee: Union[Unset, Item] = UNSET
    totals: Union[Unset, PayRunTotals] = UNSET
    period_overrides: Union[Unset, None, List[ValueOverride]] = UNSET
    totals_ytd: Union[Unset, PayRunTotals] = UNSET
    totals_ytd_overrides: Union[Unset, None, List[ValueOverride]] = UNSET
    forced_cis_vat_amount: Union[Unset, None, float] = UNSET
    holiday_accrued: Union[Unset, float] = UNSET
    state: Union[Unset, PayRunState] = UNSET
    is_closed: Union[Unset, bool] = UNSET
    manual_ni: Union[Unset, bool] = UNSET
    national_insurance_calculation: Union[Unset, NationalInsuranceCalculation] = UNSET
    payroll_code_changed: Union[Unset, bool] = UNSET
    ae_not_enroled_warning: Union[Unset, bool] = UNSET
    fps: Union[Unset, Item] = UNSET
    email_id: Union[Unset, None, str] = UNSET
    recieving_offset_pay: Union[Unset, bool] = UNSET
    payment_after_leaving: Union[Unset, bool] = UNSET
    umbrella_payment: Union[Unset, UmbrellaPayment] = UNSET
    id: Union[Unset, str] = UNSET
    personal_details: Union[Unset, PersonalDetails] = UNSET
    employment_details: Union[Unset, EmploymentDetails] = UNSET
    pay_options: Union[Unset, PayOptions] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        tax_month = self.tax_month
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        note = self.note
        bacs_sub_reference = self.bacs_sub_reference
        bacs_hash_code = self.bacs_hash_code
        percentage_of_working_days_paid_as_normal = (
            self.percentage_of_working_days_paid_as_normal
        )
        working_days_not_paid_as_normal = self.working_days_not_paid_as_normal
        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        ordinal = self.ordinal
        period = self.period
        is_new_starter = self.is_new_starter
        unpaid_absence = self.unpaid_absence
        has_attachment_orders = self.has_attachment_orders
        payment_date: Union[Unset, str] = UNSET
        if not isinstance(self.payment_date, Unset):
            payment_date = self.payment_date.isoformat()

        prior_payroll_code = self.prior_payroll_code
        pension_summary: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pension_summary, Unset):
            pension_summary = self.pension_summary.to_dict()

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        totals: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.totals, Unset):
            totals = self.totals.to_dict()

        period_overrides: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.period_overrides, Unset):
            if self.period_overrides is None:
                period_overrides = None
            else:
                period_overrides = []
                for period_overrides_item_data in self.period_overrides:
                    period_overrides_item = period_overrides_item_data.to_dict()

                    period_overrides.append(period_overrides_item)

        totals_ytd: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.totals_ytd, Unset):
            totals_ytd = self.totals_ytd.to_dict()

        totals_ytd_overrides: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.totals_ytd_overrides, Unset):
            if self.totals_ytd_overrides is None:
                totals_ytd_overrides = None
            else:
                totals_ytd_overrides = []
                for totals_ytd_overrides_item_data in self.totals_ytd_overrides:
                    totals_ytd_overrides_item = totals_ytd_overrides_item_data.to_dict()

                    totals_ytd_overrides.append(totals_ytd_overrides_item)

        forced_cis_vat_amount = self.forced_cis_vat_amount
        holiday_accrued = self.holiday_accrued
        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        is_closed = self.is_closed
        manual_ni = self.manual_ni
        national_insurance_calculation: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.national_insurance_calculation, Unset):
            national_insurance_calculation = (
                self.national_insurance_calculation.to_dict()
            )

        payroll_code_changed = self.payroll_code_changed
        ae_not_enroled_warning = self.ae_not_enroled_warning
        fps: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fps, Unset):
            fps = self.fps.to_dict()

        email_id = self.email_id
        recieving_offset_pay = self.recieving_offset_pay
        payment_after_leaving = self.payment_after_leaving
        umbrella_payment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.umbrella_payment, Unset):
            umbrella_payment = self.umbrella_payment.to_dict()

        id = self.id
        personal_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.personal_details, Unset):
            personal_details = self.personal_details.to_dict()

        employment_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employment_details, Unset):
            employment_details = self.employment_details.to_dict()

        pay_options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pay_options, Unset):
            pay_options = self.pay_options.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if tax_month is not UNSET:
            field_dict["taxMonth"] = tax_month
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if note is not UNSET:
            field_dict["note"] = note
        if bacs_sub_reference is not UNSET:
            field_dict["bacsSubReference"] = bacs_sub_reference
        if bacs_hash_code is not UNSET:
            field_dict["bacsHashCode"] = bacs_hash_code
        if percentage_of_working_days_paid_as_normal is not UNSET:
            field_dict[
                "percentageOfWorkingDaysPaidAsNormal"
            ] = percentage_of_working_days_paid_as_normal
        if working_days_not_paid_as_normal is not UNSET:
            field_dict["workingDaysNotPaidAsNormal"] = working_days_not_paid_as_normal
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if period is not UNSET:
            field_dict["period"] = period
        if is_new_starter is not UNSET:
            field_dict["isNewStarter"] = is_new_starter
        if unpaid_absence is not UNSET:
            field_dict["unpaidAbsence"] = unpaid_absence
        if has_attachment_orders is not UNSET:
            field_dict["hasAttachmentOrders"] = has_attachment_orders
        if payment_date is not UNSET:
            field_dict["paymentDate"] = payment_date
        if prior_payroll_code is not UNSET:
            field_dict["priorPayrollCode"] = prior_payroll_code
        if pension_summary is not UNSET:
            field_dict["pensionSummary"] = pension_summary
        if employee is not UNSET:
            field_dict["employee"] = employee
        if totals is not UNSET:
            field_dict["totals"] = totals
        if period_overrides is not UNSET:
            field_dict["periodOverrides"] = period_overrides
        if totals_ytd is not UNSET:
            field_dict["totalsYtd"] = totals_ytd
        if totals_ytd_overrides is not UNSET:
            field_dict["totalsYtdOverrides"] = totals_ytd_overrides
        if forced_cis_vat_amount is not UNSET:
            field_dict["forcedCisVatAmount"] = forced_cis_vat_amount
        if holiday_accrued is not UNSET:
            field_dict["holidayAccrued"] = holiday_accrued
        if state is not UNSET:
            field_dict["state"] = state
        if is_closed is not UNSET:
            field_dict["isClosed"] = is_closed
        if manual_ni is not UNSET:
            field_dict["manualNi"] = manual_ni
        if national_insurance_calculation is not UNSET:
            field_dict["nationalInsuranceCalculation"] = national_insurance_calculation
        if payroll_code_changed is not UNSET:
            field_dict["payrollCodeChanged"] = payroll_code_changed
        if ae_not_enroled_warning is not UNSET:
            field_dict["aeNotEnroledWarning"] = ae_not_enroled_warning
        if fps is not UNSET:
            field_dict["fps"] = fps
        if email_id is not UNSET:
            field_dict["emailId"] = email_id
        if recieving_offset_pay is not UNSET:
            field_dict["recievingOffsetPay"] = recieving_offset_pay
        if payment_after_leaving is not UNSET:
            field_dict["paymentAfterLeaving"] = payment_after_leaving
        if umbrella_payment is not UNSET:
            field_dict["umbrellaPayment"] = umbrella_payment
        if id is not UNSET:
            field_dict["id"] = id
        if personal_details is not UNSET:
            field_dict["personalDetails"] = personal_details
        if employment_details is not UNSET:
            field_dict["employmentDetails"] = employment_details
        if pay_options is not UNSET:
            field_dict["payOptions"] = pay_options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        tax_month = d.pop("taxMonth", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.date]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date).date()

        note = d.pop("note", UNSET)

        bacs_sub_reference = d.pop("bacsSubReference", UNSET)

        bacs_hash_code = d.pop("bacsHashCode", UNSET)

        percentage_of_working_days_paid_as_normal = d.pop(
            "percentageOfWorkingDaysPaidAsNormal", UNSET
        )

        working_days_not_paid_as_normal = d.pop("workingDaysNotPaidAsNormal", UNSET)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        ordinal = d.pop("ordinal", UNSET)

        period = d.pop("period", UNSET)

        is_new_starter = d.pop("isNewStarter", UNSET)

        unpaid_absence = d.pop("unpaidAbsence", UNSET)

        has_attachment_orders = d.pop("hasAttachmentOrders", UNSET)

        _payment_date = d.pop("paymentDate", UNSET)
        payment_date: Union[Unset, datetime.date]
        if isinstance(_payment_date, Unset):
            payment_date = UNSET
        else:
            payment_date = isoparse(_payment_date).date()

        prior_payroll_code = d.pop("priorPayrollCode", UNSET)

        _pension_summary = d.pop("pensionSummary", UNSET)
        pension_summary: Union[Unset, PensionSummary]
        if isinstance(_pension_summary, Unset):
            pension_summary = UNSET
        else:
            pension_summary = PensionSummary.from_dict(_pension_summary)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        _totals = d.pop("totals", UNSET)
        totals: Union[Unset, PayRunTotals]
        if isinstance(_totals, Unset):
            totals = UNSET
        else:
            totals = PayRunTotals.from_dict(_totals)

        period_overrides = []
        _period_overrides = d.pop("periodOverrides", UNSET)
        for period_overrides_item_data in _period_overrides or []:
            period_overrides_item = ValueOverride.from_dict(period_overrides_item_data)

            period_overrides.append(period_overrides_item)

        _totals_ytd = d.pop("totalsYtd", UNSET)
        totals_ytd: Union[Unset, PayRunTotals]
        if isinstance(_totals_ytd, Unset):
            totals_ytd = UNSET
        else:
            totals_ytd = PayRunTotals.from_dict(_totals_ytd)

        totals_ytd_overrides = []
        _totals_ytd_overrides = d.pop("totalsYtdOverrides", UNSET)
        for totals_ytd_overrides_item_data in _totals_ytd_overrides or []:
            totals_ytd_overrides_item = ValueOverride.from_dict(
                totals_ytd_overrides_item_data
            )

            totals_ytd_overrides.append(totals_ytd_overrides_item)

        forced_cis_vat_amount = d.pop("forcedCisVatAmount", UNSET)

        holiday_accrued = d.pop("holidayAccrued", UNSET)

        _state = d.pop("state", UNSET)
        state: Union[Unset, PayRunState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = PayRunState(_state)

        is_closed = d.pop("isClosed", UNSET)

        manual_ni = d.pop("manualNi", UNSET)

        _national_insurance_calculation = d.pop("nationalInsuranceCalculation", UNSET)
        national_insurance_calculation: Union[Unset, NationalInsuranceCalculation]
        if isinstance(_national_insurance_calculation, Unset):
            national_insurance_calculation = UNSET
        else:
            national_insurance_calculation = NationalInsuranceCalculation.from_dict(
                _national_insurance_calculation
            )

        payroll_code_changed = d.pop("payrollCodeChanged", UNSET)

        ae_not_enroled_warning = d.pop("aeNotEnroledWarning", UNSET)

        _fps = d.pop("fps", UNSET)
        fps: Union[Unset, Item]
        if isinstance(_fps, Unset):
            fps = UNSET
        else:
            fps = Item.from_dict(_fps)

        email_id = d.pop("emailId", UNSET)

        recieving_offset_pay = d.pop("recievingOffsetPay", UNSET)

        payment_after_leaving = d.pop("paymentAfterLeaving", UNSET)

        _umbrella_payment = d.pop("umbrellaPayment", UNSET)
        umbrella_payment: Union[Unset, UmbrellaPayment]
        if isinstance(_umbrella_payment, Unset):
            umbrella_payment = UNSET
        else:
            umbrella_payment = UmbrellaPayment.from_dict(_umbrella_payment)

        id = d.pop("id", UNSET)

        _personal_details = d.pop("personalDetails", UNSET)
        personal_details: Union[Unset, PersonalDetails]
        if isinstance(_personal_details, Unset):
            personal_details = UNSET
        else:
            personal_details = PersonalDetails.from_dict(_personal_details)

        _employment_details = d.pop("employmentDetails", UNSET)
        employment_details: Union[Unset, EmploymentDetails]
        if isinstance(_employment_details, Unset):
            employment_details = UNSET
        else:
            employment_details = EmploymentDetails.from_dict(_employment_details)

        _pay_options = d.pop("payOptions", UNSET)
        pay_options: Union[Unset, PayOptions]
        if isinstance(_pay_options, Unset):
            pay_options = UNSET
        else:
            pay_options = PayOptions.from_dict(_pay_options)

        pay_run_entry = cls(
            tax_year=tax_year,
            tax_month=tax_month,
            start_date=start_date,
            end_date=end_date,
            note=note,
            bacs_sub_reference=bacs_sub_reference,
            bacs_hash_code=bacs_hash_code,
            percentage_of_working_days_paid_as_normal=percentage_of_working_days_paid_as_normal,
            working_days_not_paid_as_normal=working_days_not_paid_as_normal,
            pay_period=pay_period,
            ordinal=ordinal,
            period=period,
            is_new_starter=is_new_starter,
            unpaid_absence=unpaid_absence,
            has_attachment_orders=has_attachment_orders,
            payment_date=payment_date,
            prior_payroll_code=prior_payroll_code,
            pension_summary=pension_summary,
            employee=employee,
            totals=totals,
            period_overrides=period_overrides,
            totals_ytd=totals_ytd,
            totals_ytd_overrides=totals_ytd_overrides,
            forced_cis_vat_amount=forced_cis_vat_amount,
            holiday_accrued=holiday_accrued,
            state=state,
            is_closed=is_closed,
            manual_ni=manual_ni,
            national_insurance_calculation=national_insurance_calculation,
            payroll_code_changed=payroll_code_changed,
            ae_not_enroled_warning=ae_not_enroled_warning,
            fps=fps,
            email_id=email_id,
            recieving_offset_pay=recieving_offset_pay,
            payment_after_leaving=payment_after_leaving,
            umbrella_payment=umbrella_payment,
            id=id,
            personal_details=personal_details,
            employment_details=employment_details,
            pay_options=pay_options,
        )

        return pay_run_entry



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="HmrcDetails")


@attr.s(auto_attribs=True)
class HmrcDetails:
    """
    Attributes:
        office_number (Union[Unset, None, str]):
        paye_reference (Union[Unset, None, str]):
        accounts_office_reference (Union[Unset, None, str]):
        econ (Union[Unset, None, str]):
        utr (Union[Unset, None, str]):
        co_tax (Union[Unset, None, str]):
        employment_allowance (Union[Unset, bool]):
        employment_allowance_max_claim (Union[Unset, float]): You might reduce this from the default if you've made/are
            making a claim in another system
        small_employers_relief (Union[Unset, bool]):
        apprenticeship_levy (Union[Unset, bool]):
        apprenticeship_levy_allowance (Union[Unset, float]):
        quarterly_payment_schedule (Union[Unset, bool]): Set to true if the employer pays HMRC on a quarterly schedule.
            A value of false implies a monthly schedule.
            <b>Warning:</b> Changing this value after starting PayRuns will
            delete any existing payments or adjustments you may have entered.
        include_employment_allowance_on_monthly_journal (Union[Unset, bool]): If the employer is not on a
            QuarterlyPaymentSchedule
            and is claiming EmploymentAllowance, then set this to true to include a line for
            Employment Allowance on the journal for the monthly schedule.
        carry_forward_unpaid_liabilities (Union[Unset, bool]): If set to true then any unpaid amounts from previous
            periods will be brought forward
            to work out the liability for the current period.
            You'd set this to false if you don't want to track payments.
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    office_number: Union[Unset, None, str] = UNSET
    paye_reference: Union[Unset, None, str] = UNSET
    accounts_office_reference: Union[Unset, None, str] = UNSET
    econ: Union[Unset, None, str] = UNSET
    utr: Union[Unset, None, str] = UNSET
    co_tax: Union[Unset, None, str] = UNSET
    employment_allowance: Union[Unset, bool] = UNSET
    employment_allowance_max_claim: Union[Unset, float] = UNSET
    small_employers_relief: Union[Unset, bool] = UNSET
    apprenticeship_levy: Union[Unset, bool] = UNSET
    apprenticeship_levy_allowance: Union[Unset, float] = UNSET
    quarterly_payment_schedule: Union[Unset, bool] = UNSET
    include_employment_allowance_on_monthly_journal: Union[Unset, bool] = UNSET
    carry_forward_unpaid_liabilities: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        office_number = self.office_number
        paye_reference = self.paye_reference
        accounts_office_reference = self.accounts_office_reference
        econ = self.econ
        utr = self.utr
        co_tax = self.co_tax
        employment_allowance = self.employment_allowance
        employment_allowance_max_claim = self.employment_allowance_max_claim
        small_employers_relief = self.small_employers_relief
        apprenticeship_levy = self.apprenticeship_levy
        apprenticeship_levy_allowance = self.apprenticeship_levy_allowance
        quarterly_payment_schedule = self.quarterly_payment_schedule
        include_employment_allowance_on_monthly_journal = (
            self.include_employment_allowance_on_monthly_journal
        )
        carry_forward_unpaid_liabilities = self.carry_forward_unpaid_liabilities
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if office_number is not UNSET:
            field_dict["officeNumber"] = office_number
        if paye_reference is not UNSET:
            field_dict["payeReference"] = paye_reference
        if accounts_office_reference is not UNSET:
            field_dict["accountsOfficeReference"] = accounts_office_reference
        if econ is not UNSET:
            field_dict["econ"] = econ
        if utr is not UNSET:
            field_dict["utr"] = utr
        if co_tax is not UNSET:
            field_dict["coTax"] = co_tax
        if employment_allowance is not UNSET:
            field_dict["employmentAllowance"] = employment_allowance
        if employment_allowance_max_claim is not UNSET:
            field_dict["employmentAllowanceMaxClaim"] = employment_allowance_max_claim
        if small_employers_relief is not UNSET:
            field_dict["smallEmployersRelief"] = small_employers_relief
        if apprenticeship_levy is not UNSET:
            field_dict["apprenticeshipLevy"] = apprenticeship_levy
        if apprenticeship_levy_allowance is not UNSET:
            field_dict["apprenticeshipLevyAllowance"] = apprenticeship_levy_allowance
        if quarterly_payment_schedule is not UNSET:
            field_dict["quarterlyPaymentSchedule"] = quarterly_payment_schedule
        if include_employment_allowance_on_monthly_journal is not UNSET:
            field_dict[
                "includeEmploymentAllowanceOnMonthlyJournal"
            ] = include_employment_allowance_on_monthly_journal
        if carry_forward_unpaid_liabilities is not UNSET:
            field_dict[
                "carryForwardUnpaidLiabilities"
            ] = carry_forward_unpaid_liabilities
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        office_number = d.pop("officeNumber", UNSET)

        paye_reference = d.pop("payeReference", UNSET)

        accounts_office_reference = d.pop("accountsOfficeReference", UNSET)

        econ = d.pop("econ", UNSET)

        utr = d.pop("utr", UNSET)

        co_tax = d.pop("coTax", UNSET)

        employment_allowance = d.pop("employmentAllowance", UNSET)

        employment_allowance_max_claim = d.pop("employmentAllowanceMaxClaim", UNSET)

        small_employers_relief = d.pop("smallEmployersRelief", UNSET)

        apprenticeship_levy = d.pop("apprenticeshipLevy", UNSET)

        apprenticeship_levy_allowance = d.pop("apprenticeshipLevyAllowance", UNSET)

        quarterly_payment_schedule = d.pop("quarterlyPaymentSchedule", UNSET)

        include_employment_allowance_on_monthly_journal = d.pop(
            "includeEmploymentAllowanceOnMonthlyJournal", UNSET
        )

        carry_forward_unpaid_liabilities = d.pop("carryForwardUnpaidLiabilities", UNSET)

        id = d.pop("id", UNSET)

        hmrc_details = cls(
            office_number=office_number,
            paye_reference=paye_reference,
            accounts_office_reference=accounts_office_reference,
            econ=econ,
            utr=utr,
            co_tax=co_tax,
            employment_allowance=employment_allowance,
            employment_allowance_max_claim=employment_allowance_max_claim,
            small_employers_relief=small_employers_relief,
            apprenticeship_levy=apprenticeship_levy,
            apprenticeship_levy_allowance=apprenticeship_levy_allowance,
            quarterly_payment_schedule=quarterly_payment_schedule,
            include_employment_allowance_on_monthly_journal=include_employment_allowance_on_monthly_journal,
            carry_forward_unpaid_liabilities=carry_forward_unpaid_liabilities,
            id=id,
        )

        return hmrc_details



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.hours_normally_worked import HoursNormallyWorked
from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsFields")


@attr.s(auto_attribs=True)
class FpsFields:
    """
    Attributes:
        off_payroll_worker (Union[Unset, bool]):
        irregular_payment_pattern (Union[Unset, bool]): True if employee is currently on an irregular payment patter
        non_individual (Union[Unset, bool]): True if Employee's payments are being made to a body (eg, trustee,
            corporate organisation or personal representative)
        hours_normally_worked (Union[Unset, HoursNormallyWorked]):
    """

    off_payroll_worker: Union[Unset, bool] = UNSET
    irregular_payment_pattern: Union[Unset, bool] = UNSET
    non_individual: Union[Unset, bool] = UNSET
    hours_normally_worked: Union[Unset, HoursNormallyWorked] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        off_payroll_worker = self.off_payroll_worker
        irregular_payment_pattern = self.irregular_payment_pattern
        non_individual = self.non_individual
        hours_normally_worked: Union[Unset, str] = UNSET
        if not isinstance(self.hours_normally_worked, Unset):
            hours_normally_worked = self.hours_normally_worked.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if off_payroll_worker is not UNSET:
            field_dict["offPayrollWorker"] = off_payroll_worker
        if irregular_payment_pattern is not UNSET:
            field_dict["irregularPaymentPattern"] = irregular_payment_pattern
        if non_individual is not UNSET:
            field_dict["nonIndividual"] = non_individual
        if hours_normally_worked is not UNSET:
            field_dict["hoursNormallyWorked"] = hours_normally_worked

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        off_payroll_worker = d.pop("offPayrollWorker", UNSET)

        irregular_payment_pattern = d.pop("irregularPaymentPattern", UNSET)

        non_individual = d.pop("nonIndividual", UNSET)

        _hours_normally_worked = d.pop("hoursNormallyWorked", UNSET)
        hours_normally_worked: Union[Unset, HoursNormallyWorked]
        if isinstance(_hours_normally_worked, Unset):
            hours_normally_worked = UNSET
        else:
            hours_normally_worked = HoursNormallyWorked(_hours_normally_worked)

        fps_fields = cls(
            off_payroll_worker=off_payroll_worker,
            irregular_payment_pattern=irregular_payment_pattern,
            non_individual=non_individual,
            hours_normally_worked=hours_normally_worked,
        )

        return fps_fields



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.fps_employee_figs_to_date import FpsEmployeeFigsToDate
from ..models.fps_employee_n_iletters_and_values import FpsEmployeeNIlettersAndValues
from ..models.fps_employee_payment import FpsEmployeePayment
from ..models.fps_employee_starter import FpsEmployeeStarter
from ..models.fps_employer_pay_id_changed import FpsEmployerPayIdChanged
from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployment")


@attr.s(auto_attribs=True)
class FpsEmployment:
    """
    Attributes:
        employee_current_ni_letter (Union[Unset, None, str]):
        off_payroll_worker (Union[Unset, None, str]):
        occ_pen_ind (Union[Unset, None, str]):
        directors_nic (Union[Unset, None, str]):
        tax_wk_of_appt_of_director (Union[Unset, None, str]):
        starter (Union[Unset, FpsEmployeeStarter]):
        pay_id (Union[Unset, None, str]):
        pay_id_chgd (Union[Unset, FpsEmployerPayIdChanged]):
        payment_to_a_non_individual (Union[Unset, None, str]):
        irr_emp (Union[Unset, None, str]):
        leaving_date (Union[Unset, None, str]):
        figures_to_date (Union[Unset, FpsEmployeeFigsToDate]):
        payment (Union[Unset, FpsEmployeePayment]):
        n_iletters_and_values (Union[Unset, None, List[FpsEmployeeNIlettersAndValues]]):
    """

    employee_current_ni_letter: Union[Unset, None, str] = UNSET
    off_payroll_worker: Union[Unset, None, str] = UNSET
    occ_pen_ind: Union[Unset, None, str] = UNSET
    directors_nic: Union[Unset, None, str] = UNSET
    tax_wk_of_appt_of_director: Union[Unset, None, str] = UNSET
    starter: Union[Unset, FpsEmployeeStarter] = UNSET
    pay_id: Union[Unset, None, str] = UNSET
    pay_id_chgd: Union[Unset, FpsEmployerPayIdChanged] = UNSET
    payment_to_a_non_individual: Union[Unset, None, str] = UNSET
    irr_emp: Union[Unset, None, str] = UNSET
    leaving_date: Union[Unset, None, str] = UNSET
    figures_to_date: Union[Unset, FpsEmployeeFigsToDate] = UNSET
    payment: Union[Unset, FpsEmployeePayment] = UNSET
    n_iletters_and_values: Union[
        Unset, None, List[FpsEmployeeNIlettersAndValues]
    ] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee_current_ni_letter = self.employee_current_ni_letter
        off_payroll_worker = self.off_payroll_worker
        occ_pen_ind = self.occ_pen_ind
        directors_nic = self.directors_nic
        tax_wk_of_appt_of_director = self.tax_wk_of_appt_of_director
        starter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.starter, Unset):
            starter = self.starter.to_dict()

        pay_id = self.pay_id
        pay_id_chgd: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pay_id_chgd, Unset):
            pay_id_chgd = self.pay_id_chgd.to_dict()

        payment_to_a_non_individual = self.payment_to_a_non_individual
        irr_emp = self.irr_emp
        leaving_date = self.leaving_date
        figures_to_date: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.figures_to_date, Unset):
            figures_to_date = self.figures_to_date.to_dict()

        payment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.payment, Unset):
            payment = self.payment.to_dict()

        n_iletters_and_values: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.n_iletters_and_values, Unset):
            if self.n_iletters_and_values is None:
                n_iletters_and_values = None
            else:
                n_iletters_and_values = []
                for n_iletters_and_values_item_data in self.n_iletters_and_values:
                    n_iletters_and_values_item = (
                        n_iletters_and_values_item_data.to_dict()
                    )

                    n_iletters_and_values.append(n_iletters_and_values_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee_current_ni_letter is not UNSET:
            field_dict["employeeCurrentNiLetter"] = employee_current_ni_letter
        if off_payroll_worker is not UNSET:
            field_dict["offPayrollWorker"] = off_payroll_worker
        if occ_pen_ind is not UNSET:
            field_dict["occPenInd"] = occ_pen_ind
        if directors_nic is not UNSET:
            field_dict["directorsNIC"] = directors_nic
        if tax_wk_of_appt_of_director is not UNSET:
            field_dict["taxWkOfApptOfDirector"] = tax_wk_of_appt_of_director
        if starter is not UNSET:
            field_dict["starter"] = starter
        if pay_id is not UNSET:
            field_dict["payId"] = pay_id
        if pay_id_chgd is not UNSET:
            field_dict["payIdChgd"] = pay_id_chgd
        if payment_to_a_non_individual is not UNSET:
            field_dict["paymentToANonIndividual"] = payment_to_a_non_individual
        if irr_emp is not UNSET:
            field_dict["irrEmp"] = irr_emp
        if leaving_date is not UNSET:
            field_dict["leavingDate"] = leaving_date
        if figures_to_date is not UNSET:
            field_dict["figuresToDate"] = figures_to_date
        if payment is not UNSET:
            field_dict["payment"] = payment
        if n_iletters_and_values is not UNSET:
            field_dict["nIlettersAndValues"] = n_iletters_and_values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee_current_ni_letter = d.pop("employeeCurrentNiLetter", UNSET)

        off_payroll_worker = d.pop("offPayrollWorker", UNSET)

        occ_pen_ind = d.pop("occPenInd", UNSET)

        directors_nic = d.pop("directorsNIC", UNSET)

        tax_wk_of_appt_of_director = d.pop("taxWkOfApptOfDirector", UNSET)

        _starter = d.pop("starter", UNSET)
        starter: Union[Unset, FpsEmployeeStarter]
        if isinstance(_starter, Unset):
            starter = UNSET
        else:
            starter = FpsEmployeeStarter.from_dict(_starter)

        pay_id = d.pop("payId", UNSET)

        _pay_id_chgd = d.pop("payIdChgd", UNSET)
        pay_id_chgd: Union[Unset, FpsEmployerPayIdChanged]
        if isinstance(_pay_id_chgd, Unset):
            pay_id_chgd = UNSET
        else:
            pay_id_chgd = FpsEmployerPayIdChanged.from_dict(_pay_id_chgd)

        payment_to_a_non_individual = d.pop("paymentToANonIndividual", UNSET)

        irr_emp = d.pop("irrEmp", UNSET)

        leaving_date = d.pop("leavingDate", UNSET)

        _figures_to_date = d.pop("figuresToDate", UNSET)
        figures_to_date: Union[Unset, FpsEmployeeFigsToDate]
        if isinstance(_figures_to_date, Unset):
            figures_to_date = UNSET
        else:
            figures_to_date = FpsEmployeeFigsToDate.from_dict(_figures_to_date)

        _payment = d.pop("payment", UNSET)
        payment: Union[Unset, FpsEmployeePayment]
        if isinstance(_payment, Unset):
            payment = UNSET
        else:
            payment = FpsEmployeePayment.from_dict(_payment)

        n_iletters_and_values = []
        _n_iletters_and_values = d.pop("nIlettersAndValues", UNSET)
        for n_iletters_and_values_item_data in _n_iletters_and_values or []:
            n_iletters_and_values_item = FpsEmployeeNIlettersAndValues.from_dict(
                n_iletters_and_values_item_data
            )

            n_iletters_and_values.append(n_iletters_and_values_item)

        fps_employment = cls(
            employee_current_ni_letter=employee_current_ni_letter,
            off_payroll_worker=off_payroll_worker,
            occ_pen_ind=occ_pen_ind,
            directors_nic=directors_nic,
            tax_wk_of_appt_of_director=tax_wk_of_appt_of_director,
            starter=starter,
            pay_id=pay_id,
            pay_id_chgd=pay_id_chgd,
            payment_to_a_non_individual=payment_to_a_non_individual,
            irr_emp=irr_emp,
            leaving_date=leaving_date,
            figures_to_date=figures_to_date,
            payment=payment,
            n_iletters_and_values=n_iletters_and_values,
        )

        return fps_employment



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NationalInsuranceCalculationPeriodValues")


@attr.s(auto_attribs=True)
class NationalInsuranceCalculationPeriodValues:
    """Part of the TaxYearConfig that our engine uses to calculate National Insurance Contributions.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            lel (Union[Unset, float]): [readonly] Lower Earnings Limit
            pt (Union[Unset, float]): [readonly] Primary Threshold
            st (Union[Unset, float]): [readonly] Secondary Threshold
            fust (Union[Unset, float]): [readonly] Freeports Upper Accrual Threshold
            uap (Union[Unset, float]): [readonly] Upper Accrual Point
            ust (Union[Unset, float]): [readonly] Upper Secondary Threshold (under 21)
            aust (Union[Unset, float]): [readonly] Apprentice Upper Secondary Threshold (apprentice under 25)
            uel (Union[Unset, float]): [readonly] Upper Earnings Limit
            vust (Union[Unset, float]): [readonly] Veterian Upper Secondary Threshold
    """

    lel: Union[Unset, float] = UNSET
    pt: Union[Unset, float] = UNSET
    st: Union[Unset, float] = UNSET
    fust: Union[Unset, float] = UNSET
    uap: Union[Unset, float] = UNSET
    ust: Union[Unset, float] = UNSET
    aust: Union[Unset, float] = UNSET
    uel: Union[Unset, float] = UNSET
    vust: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        lel = self.lel
        pt = self.pt
        st = self.st
        fust = self.fust
        uap = self.uap
        ust = self.ust
        aust = self.aust
        uel = self.uel
        vust = self.vust

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if lel is not UNSET:
            field_dict["lel"] = lel
        if pt is not UNSET:
            field_dict["pt"] = pt
        if st is not UNSET:
            field_dict["st"] = st
        if fust is not UNSET:
            field_dict["fust"] = fust
        if uap is not UNSET:
            field_dict["uap"] = uap
        if ust is not UNSET:
            field_dict["ust"] = ust
        if aust is not UNSET:
            field_dict["aust"] = aust
        if uel is not UNSET:
            field_dict["uel"] = uel
        if vust is not UNSET:
            field_dict["vust"] = vust

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        lel = d.pop("lel", UNSET)

        pt = d.pop("pt", UNSET)

        st = d.pop("st", UNSET)

        fust = d.pop("fust", UNSET)

        uap = d.pop("uap", UNSET)

        ust = d.pop("ust", UNSET)

        aust = d.pop("aust", UNSET)

        uel = d.pop("uel", UNSET)

        vust = d.pop("vust", UNSET)

        national_insurance_calculation_period_values = cls(
            lel=lel,
            pt=pt,
            st=st,
            fust=fust,
            uap=uap,
            ust=ust,
            aust=aust,
            uel=uel,
            vust=vust,
        )

        return national_insurance_calculation_period_values



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..models.nic_summary import NicSummary
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="OpeningBalances")


@attr.s(auto_attribs=True)
class OpeningBalances:
    """
    Attributes:
        tax_year (Union[Unset, TaxYear]):
        not_new_starter (Union[Unset, bool]):
        previous_employer_gross (Union[Unset, float]):
        previous_employer_tax (Union[Unset, float]):
        current_employer_gross (Union[Unset, float]):
        current_employer_tax (Union[Unset, float]):
        current_employer_net (Union[Unset, float]):
        student_loan_deductions (Union[Unset, float]):
        postgrad_loan_deductions (Union[Unset, float]):
        empee_pen_contribns_paid (Union[Unset, float]):
        empee_pen_contribns_not_paid (Union[Unset, float]):
        smp (Union[Unset, float]):
        spp (Union[Unset, float]):
        sap (Union[Unset, float]):
        shpp (Union[Unset, float]):
        spbp (Union[Unset, float]):
        real_time_class_1a_ni (Union[Unset, float]):
        termination_payments (Union[Unset, float]):
        maps_miles (Union[Unset, int]):
        benefits_taxed_via_payroll (Union[Unset, float]):
        nic_summaries (Union[Unset, None, List[NicSummary]]):
        employee (Union[Unset, Item]):
    """

    tax_year: Union[Unset, TaxYear] = UNSET
    not_new_starter: Union[Unset, bool] = UNSET
    previous_employer_gross: Union[Unset, float] = UNSET
    previous_employer_tax: Union[Unset, float] = UNSET
    current_employer_gross: Union[Unset, float] = UNSET
    current_employer_tax: Union[Unset, float] = UNSET
    current_employer_net: Union[Unset, float] = UNSET
    student_loan_deductions: Union[Unset, float] = UNSET
    postgrad_loan_deductions: Union[Unset, float] = UNSET
    empee_pen_contribns_paid: Union[Unset, float] = UNSET
    empee_pen_contribns_not_paid: Union[Unset, float] = UNSET
    smp: Union[Unset, float] = UNSET
    spp: Union[Unset, float] = UNSET
    sap: Union[Unset, float] = UNSET
    shpp: Union[Unset, float] = UNSET
    spbp: Union[Unset, float] = UNSET
    real_time_class_1a_ni: Union[Unset, float] = UNSET
    termination_payments: Union[Unset, float] = UNSET
    maps_miles: Union[Unset, int] = UNSET
    benefits_taxed_via_payroll: Union[Unset, float] = UNSET
    nic_summaries: Union[Unset, None, List[NicSummary]] = UNSET
    employee: Union[Unset, Item] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        not_new_starter = self.not_new_starter
        previous_employer_gross = self.previous_employer_gross
        previous_employer_tax = self.previous_employer_tax
        current_employer_gross = self.current_employer_gross
        current_employer_tax = self.current_employer_tax
        current_employer_net = self.current_employer_net
        student_loan_deductions = self.student_loan_deductions
        postgrad_loan_deductions = self.postgrad_loan_deductions
        empee_pen_contribns_paid = self.empee_pen_contribns_paid
        empee_pen_contribns_not_paid = self.empee_pen_contribns_not_paid
        smp = self.smp
        spp = self.spp
        sap = self.sap
        shpp = self.shpp
        spbp = self.spbp
        real_time_class_1a_ni = self.real_time_class_1a_ni
        termination_payments = self.termination_payments
        maps_miles = self.maps_miles
        benefits_taxed_via_payroll = self.benefits_taxed_via_payroll
        nic_summaries: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.nic_summaries, Unset):
            if self.nic_summaries is None:
                nic_summaries = None
            else:
                nic_summaries = []
                for nic_summaries_item_data in self.nic_summaries:
                    nic_summaries_item = nic_summaries_item_data.to_dict()

                    nic_summaries.append(nic_summaries_item)

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if not_new_starter is not UNSET:
            field_dict["notNewStarter"] = not_new_starter
        if previous_employer_gross is not UNSET:
            field_dict["previousEmployerGross"] = previous_employer_gross
        if previous_employer_tax is not UNSET:
            field_dict["previousEmployerTax"] = previous_employer_tax
        if current_employer_gross is not UNSET:
            field_dict["currentEmployerGross"] = current_employer_gross
        if current_employer_tax is not UNSET:
            field_dict["currentEmployerTax"] = current_employer_tax
        if current_employer_net is not UNSET:
            field_dict["currentEmployerNet"] = current_employer_net
        if student_loan_deductions is not UNSET:
            field_dict["studentLoanDeductions"] = student_loan_deductions
        if postgrad_loan_deductions is not UNSET:
            field_dict["postgradLoanDeductions"] = postgrad_loan_deductions
        if empee_pen_contribns_paid is not UNSET:
            field_dict["empeePenContribnsPaid"] = empee_pen_contribns_paid
        if empee_pen_contribns_not_paid is not UNSET:
            field_dict["empeePenContribnsNotPaid"] = empee_pen_contribns_not_paid
        if smp is not UNSET:
            field_dict["smp"] = smp
        if spp is not UNSET:
            field_dict["spp"] = spp
        if sap is not UNSET:
            field_dict["sap"] = sap
        if shpp is not UNSET:
            field_dict["shpp"] = shpp
        if spbp is not UNSET:
            field_dict["spbp"] = spbp
        if real_time_class_1a_ni is not UNSET:
            field_dict["realTimeClass1ANi"] = real_time_class_1a_ni
        if termination_payments is not UNSET:
            field_dict["terminationPayments"] = termination_payments
        if maps_miles is not UNSET:
            field_dict["mapsMiles"] = maps_miles
        if benefits_taxed_via_payroll is not UNSET:
            field_dict["benefitsTaxedViaPayroll"] = benefits_taxed_via_payroll
        if nic_summaries is not UNSET:
            field_dict["nicSummaries"] = nic_summaries
        if employee is not UNSET:
            field_dict["employee"] = employee

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        not_new_starter = d.pop("notNewStarter", UNSET)

        previous_employer_gross = d.pop("previousEmployerGross", UNSET)

        previous_employer_tax = d.pop("previousEmployerTax", UNSET)

        current_employer_gross = d.pop("currentEmployerGross", UNSET)

        current_employer_tax = d.pop("currentEmployerTax", UNSET)

        current_employer_net = d.pop("currentEmployerNet", UNSET)

        student_loan_deductions = d.pop("studentLoanDeductions", UNSET)

        postgrad_loan_deductions = d.pop("postgradLoanDeductions", UNSET)

        empee_pen_contribns_paid = d.pop("empeePenContribnsPaid", UNSET)

        empee_pen_contribns_not_paid = d.pop("empeePenContribnsNotPaid", UNSET)

        smp = d.pop("smp", UNSET)

        spp = d.pop("spp", UNSET)

        sap = d.pop("sap", UNSET)

        shpp = d.pop("shpp", UNSET)

        spbp = d.pop("spbp", UNSET)

        real_time_class_1a_ni = d.pop("realTimeClass1ANi", UNSET)

        termination_payments = d.pop("terminationPayments", UNSET)

        maps_miles = d.pop("mapsMiles", UNSET)

        benefits_taxed_via_payroll = d.pop("benefitsTaxedViaPayroll", UNSET)

        nic_summaries = []
        _nic_summaries = d.pop("nicSummaries", UNSET)
        for nic_summaries_item_data in _nic_summaries or []:
            nic_summaries_item = NicSummary.from_dict(nic_summaries_item_data)

            nic_summaries.append(nic_summaries_item)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        opening_balances = cls(
            tax_year=tax_year,
            not_new_starter=not_new_starter,
            previous_employer_gross=previous_employer_gross,
            previous_employer_tax=previous_employer_tax,
            current_employer_gross=current_employer_gross,
            current_employer_tax=current_employer_tax,
            current_employer_net=current_employer_net,
            student_loan_deductions=student_loan_deductions,
            postgrad_loan_deductions=postgrad_loan_deductions,
            empee_pen_contribns_paid=empee_pen_contribns_paid,
            empee_pen_contribns_not_paid=empee_pen_contribns_not_paid,
            smp=smp,
            spp=spp,
            sap=sap,
            shpp=shpp,
            spbp=spbp,
            real_time_class_1a_ni=real_time_class_1a_ni,
            termination_payments=termination_payments,
            maps_miles=maps_miles,
            benefits_taxed_via_payroll=benefits_taxed_via_payroll,
            nic_summaries=nic_summaries,
            employee=employee,
        )

        return opening_balances



from enum import Enum


class AeStatutoryLetter(str, Enum):
    B1 = "B1"
    B2 = "B2"
    B3 = "B3"
    B4 = "B4"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.rti_validation_warning_type import RtiValidationWarningType
from ..types import UNSET, Unset

T = TypeVar("T", bound="RtiValidationWarning")


@attr.s(auto_attribs=True)
class RtiValidationWarning:
    """
    Attributes:
        type (Union[Unset, RtiValidationWarningType]):
        employee_id (Union[Unset, None, str]):
    """

    type: Union[Unset, RtiValidationWarningType] = UNSET
    employee_id: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        employee_id = self.employee_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if employee_id is not UNSET:
            field_dict["employeeId"] = employee_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, RtiValidationWarningType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = RtiValidationWarningType(_type)

        employee_id = d.pop("employeeId", UNSET)

        rti_validation_warning = cls(
            type=type,
            employee_id=employee_id,
        )

        return rti_validation_warning



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.exb_p11_db_class_1a_adjustment import ExbP11DbClass1AAdjustment
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExbP11DbClass1AAdjustments")


@attr.s(auto_attribs=True)
class ExbP11DbClass1AAdjustments:
    """
    Attributes:
        total_benefit (Union[Unset, None, str]):
        amount_due (Union[Unset, ExbP11DbClass1AAdjustment]):
        amount_not_due (Union[Unset, ExbP11DbClass1AAdjustment]):
        total (Union[Unset, None, str]):
        payable (Union[Unset, None, str]):
    """

    total_benefit: Union[Unset, None, str] = UNSET
    amount_due: Union[Unset, ExbP11DbClass1AAdjustment] = UNSET
    amount_not_due: Union[Unset, ExbP11DbClass1AAdjustment] = UNSET
    total: Union[Unset, None, str] = UNSET
    payable: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        total_benefit = self.total_benefit
        amount_due: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.amount_due, Unset):
            amount_due = self.amount_due.to_dict()

        amount_not_due: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.amount_not_due, Unset):
            amount_not_due = self.amount_not_due.to_dict()

        total = self.total
        payable = self.payable

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if total_benefit is not UNSET:
            field_dict["totalBenefit"] = total_benefit
        if amount_due is not UNSET:
            field_dict["amountDue"] = amount_due
        if amount_not_due is not UNSET:
            field_dict["amountNotDue"] = amount_not_due
        if total is not UNSET:
            field_dict["total"] = total
        if payable is not UNSET:
            field_dict["payable"] = payable

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        total_benefit = d.pop("totalBenefit", UNSET)

        _amount_due = d.pop("amountDue", UNSET)
        amount_due: Union[Unset, ExbP11DbClass1AAdjustment]
        if isinstance(_amount_due, Unset):
            amount_due = UNSET
        else:
            amount_due = ExbP11DbClass1AAdjustment.from_dict(_amount_due)

        _amount_not_due = d.pop("amountNotDue", UNSET)
        amount_not_due: Union[Unset, ExbP11DbClass1AAdjustment]
        if isinstance(_amount_not_due, Unset):
            amount_not_due = UNSET
        else:
            amount_not_due = ExbP11DbClass1AAdjustment.from_dict(_amount_not_due)

        total = d.pop("total", UNSET)

        payable = d.pop("payable", UNSET)

        exb_p11_db_class_1a_adjustments = cls(
            total_benefit=total_benefit,
            amount_due=amount_due,
            amount_not_due=amount_not_due,
            total=total,
            payable=payable,
        )

        return exb_p11_db_class_1a_adjustments



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.cis_statement import CisStatement
from ..types import UNSET, Unset

T = TypeVar("T", bound="CisStatementListReportResponse")


@attr.s(auto_attribs=True)
class CisStatementListReportResponse:
    """Used to encapsulate a response for any of the reports.
    See the Introduction Guide for Reports for more details

        Attributes:
            type (Union[Unset, None, str]): [readonly] The content-type, this would usually be the same as the accept header
                you provided when you requested the report
            content (Union[Unset, None, str]): [readonly] This could contain a link to a PDF file, HTML content or other
                content, depending on the Type value.
            model (Union[Unset, None, List[CisStatement]]): [readonly] If the type is application.json then this will
                contain a JSON representation of the relevant model
    """

    type: Union[Unset, None, str] = UNSET
    content: Union[Unset, None, str] = UNSET
    model: Union[Unset, None, List[CisStatement]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        content = self.content
        model: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.model, Unset):
            if self.model is None:
                model = None
            else:
                model = []
                for model_item_data in self.model:
                    model_item = model_item_data.to_dict()

                    model.append(model_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if content is not UNSET:
            field_dict["content"] = content
        if model is not UNSET:
            field_dict["model"] = model

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        content = d.pop("content", UNSET)

        model = []
        _model = d.pop("model", UNSET)
        for model_item_data in _model or []:
            model_item = CisStatement.from_dict(model_item_data)

            model.append(model_item)

        cis_statement_list_report_response = cls(
            type=type,
            content=content,
            model=model,
        )

        return cis_statement_list_report_response



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="UmbrellaReconciliationReportLine")


@attr.s(auto_attribs=True)
class UmbrellaReconciliationReportLine:
    """
    Attributes:
        employee (Union[Unset, Item]):
        period (Union[Unset, None, str]):
        payroll_code (Union[Unset, None, str]):
        department (Union[Unset, None, str]):
        total_gross (Union[Unset, float]):
        net_pay (Union[Unset, float]):
        tax (Union[Unset, float]):
        employee_ni (Union[Unset, float]):
        employer_ni (Union[Unset, float]):
        employee_pension (Union[Unset, float]):
        employer_pension (Union[Unset, float]):
        expenses (Union[Unset, float]):
        fee (Union[Unset, float]):
        app_levy (Union[Unset, float]):
        invoice_value (Union[Unset, float]):
        employee_costs (Union[Unset, float]):
        employer_costs (Union[Unset, float]):
        total (Union[Unset, float]):
    """

    employee: Union[Unset, Item] = UNSET
    period: Union[Unset, None, str] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    department: Union[Unset, None, str] = UNSET
    total_gross: Union[Unset, float] = UNSET
    net_pay: Union[Unset, float] = UNSET
    tax: Union[Unset, float] = UNSET
    employee_ni: Union[Unset, float] = UNSET
    employer_ni: Union[Unset, float] = UNSET
    employee_pension: Union[Unset, float] = UNSET
    employer_pension: Union[Unset, float] = UNSET
    expenses: Union[Unset, float] = UNSET
    fee: Union[Unset, float] = UNSET
    app_levy: Union[Unset, float] = UNSET
    invoice_value: Union[Unset, float] = UNSET
    employee_costs: Union[Unset, float] = UNSET
    employer_costs: Union[Unset, float] = UNSET
    total: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        period = self.period
        payroll_code = self.payroll_code
        department = self.department
        total_gross = self.total_gross
        net_pay = self.net_pay
        tax = self.tax
        employee_ni = self.employee_ni
        employer_ni = self.employer_ni
        employee_pension = self.employee_pension
        employer_pension = self.employer_pension
        expenses = self.expenses
        fee = self.fee
        app_levy = self.app_levy
        invoice_value = self.invoice_value
        employee_costs = self.employee_costs
        employer_costs = self.employer_costs
        total = self.total

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if period is not UNSET:
            field_dict["period"] = period
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if department is not UNSET:
            field_dict["department"] = department
        if total_gross is not UNSET:
            field_dict["totalGross"] = total_gross
        if net_pay is not UNSET:
            field_dict["netPay"] = net_pay
        if tax is not UNSET:
            field_dict["tax"] = tax
        if employee_ni is not UNSET:
            field_dict["employeeNi"] = employee_ni
        if employer_ni is not UNSET:
            field_dict["employerNi"] = employer_ni
        if employee_pension is not UNSET:
            field_dict["employeePension"] = employee_pension
        if employer_pension is not UNSET:
            field_dict["employerPension"] = employer_pension
        if expenses is not UNSET:
            field_dict["expenses"] = expenses
        if fee is not UNSET:
            field_dict["fee"] = fee
        if app_levy is not UNSET:
            field_dict["appLevy"] = app_levy
        if invoice_value is not UNSET:
            field_dict["invoiceValue"] = invoice_value
        if employee_costs is not UNSET:
            field_dict["employeeCosts"] = employee_costs
        if employer_costs is not UNSET:
            field_dict["employerCosts"] = employer_costs
        if total is not UNSET:
            field_dict["total"] = total

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        period = d.pop("period", UNSET)

        payroll_code = d.pop("payrollCode", UNSET)

        department = d.pop("department", UNSET)

        total_gross = d.pop("totalGross", UNSET)

        net_pay = d.pop("netPay", UNSET)

        tax = d.pop("tax", UNSET)

        employee_ni = d.pop("employeeNi", UNSET)

        employer_ni = d.pop("employerNi", UNSET)

        employee_pension = d.pop("employeePension", UNSET)

        employer_pension = d.pop("employerPension", UNSET)

        expenses = d.pop("expenses", UNSET)

        fee = d.pop("fee", UNSET)

        app_levy = d.pop("appLevy", UNSET)

        invoice_value = d.pop("invoiceValue", UNSET)

        employee_costs = d.pop("employeeCosts", UNSET)

        employer_costs = d.pop("employerCosts", UNSET)

        total = d.pop("total", UNSET)

        umbrella_reconciliation_report_line = cls(
            employee=employee,
            period=period,
            payroll_code=payroll_code,
            department=department,
            total_gross=total_gross,
            net_pay=net_pay,
            tax=tax,
            employee_ni=employee_ni,
            employer_ni=employer_ni,
            employee_pension=employee_pension,
            employer_pension=employer_pension,
            expenses=expenses,
            fee=fee,
            app_levy=app_levy,
            invoice_value=invoice_value,
            employee_costs=employee_costs,
            employer_costs=employer_costs,
            total=total,
        )

        return umbrella_reconciliation_report_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.cost_breakdown import CostBreakdown
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRunCostSummary")


@attr.s(auto_attribs=True)
class PayRunCostSummary:
    """Represents the various components that make up the costs of a PayRun

    Attributes:
        tax (Union[Unset, CostBreakdown]): Represents the breakdown of a PayRun cost.
            The Value if calculated using the breakdown of the cost.
        national_insurance (Union[Unset, CostBreakdown]): Represents the breakdown of a PayRun cost.
            The Value if calculated using the breakdown of the cost.
    """

    tax: Union[Unset, CostBreakdown] = UNSET
    national_insurance: Union[Unset, CostBreakdown] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        tax: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tax, Unset):
            tax = self.tax.to_dict()

        national_insurance: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.national_insurance, Unset):
            national_insurance = self.national_insurance.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if tax is not UNSET:
            field_dict["tax"] = tax
        if national_insurance is not UNSET:
            field_dict["nationalInsurance"] = national_insurance

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _tax = d.pop("tax", UNSET)
        tax: Union[Unset, CostBreakdown]
        if isinstance(_tax, Unset):
            tax = UNSET
        else:
            tax = CostBreakdown.from_dict(_tax)

        _national_insurance = d.pop("nationalInsurance", UNSET)
        national_insurance: Union[Unset, CostBreakdown]
        if isinstance(_national_insurance, Unset):
            national_insurance = UNSET
        else:
            national_insurance = CostBreakdown.from_dict(_national_insurance)

        pay_run_cost_summary = cls(
            tax=tax,
            national_insurance=national_insurance,
        )

        return pay_run_cost_summary



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.papdis_employer import PapdisEmployer
from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisPensionProvider")


@attr.s(auto_attribs=True)
class PapdisPensionProvider:
    """
    Attributes:
        pension_provider_id (Union[Unset, None, str]): [readonly] Taken from the papdisProviderId property of the
            PensionProvider
        employers (Union[Unset, None, List[PapdisEmployer]]):
    """

    pension_provider_id: Union[Unset, None, str] = UNSET
    employers: Union[Unset, None, List[PapdisEmployer]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pension_provider_id = self.pension_provider_id
        employers: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.employers, Unset):
            if self.employers is None:
                employers = None
            else:
                employers = []
                for employers_item_data in self.employers:
                    employers_item = employers_item_data.to_dict()

                    employers.append(employers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pension_provider_id is not UNSET:
            field_dict["pensionProviderId"] = pension_provider_id
        if employers is not UNSET:
            field_dict["employers"] = employers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pension_provider_id = d.pop("pensionProviderId", UNSET)

        employers = []
        _employers = d.pop("employers", UNSET)
        for employers_item_data in _employers or []:
            employers_item = PapdisEmployer.from_dict(employers_item_data)

            employers.append(employers_item)

        papdis_pension_provider = cls(
            pension_provider_id=pension_provider_id,
            employers=employers,
        )

        return papdis_pension_provider



from typing import Any, Dict, Optional, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.national_insurance_calculation import NationalInsuranceCalculation
from ...models.pay_periods import PayPeriods
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    ni_category: Union[Unset, None, str] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
) -> Dict[str, Any]:
    url = "{}/engine/ni".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_tax_year: Union[Unset, None, str] = UNSET
    if not isinstance(tax_year, Unset):
        json_tax_year = tax_year.value if tax_year else None

    params["taxYear"] = json_tax_year

    params["gross"] = gross

    params["niCategory"] = ni_category

    json_pay_period: Union[Unset, None, str] = UNSET
    if not isinstance(pay_period, Unset):
        json_pay_period = pay_period.value if pay_period else None

    params["payPeriod"] = json_pay_period

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[NationalInsuranceCalculation]:
    if response.status_code == 200:
        response_200 = NationalInsuranceCalculation.from_dict(response.json())

        return response_200
    return raise_staffology_exception(response)


def _build_response(
    *, response: httpx.Response
) -> Response[NationalInsuranceCalculation]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    ni_category: Union[Unset, None, str] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
) -> Response[NationalInsuranceCalculation]:
    """Calculate NI due.

     Calculates National Insurance Contributions due given the values specified.
    You would never need to use this API call in practice. It is provided just for information and
    testing purposes.
    Access is limited so you'll probably receive a 401 response if you try to use it.

    Args:
        tax_year (Union[Unset, None, TaxYear]):
        gross (Union[Unset, None, float]):
        ni_category (Union[Unset, None, str]):
        pay_period (Union[Unset, None, PayPeriods]):

    Returns:
        Response[NationalInsuranceCalculation]
    """

    kwargs = _get_kwargs(
        client=client,
        tax_year=tax_year,
        gross=gross,
        ni_category=ni_category,
        pay_period=pay_period,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    ni_category: Union[Unset, None, str] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
) -> Optional[NationalInsuranceCalculation]:
    """Calculate NI due.

     Calculates National Insurance Contributions due given the values specified.
    You would never need to use this API call in practice. It is provided just for information and
    testing purposes.
    Access is limited so you'll probably receive a 401 response if you try to use it.

    Args:
        tax_year (Union[Unset, None, TaxYear]):
        gross (Union[Unset, None, float]):
        ni_category (Union[Unset, None, str]):
        pay_period (Union[Unset, None, PayPeriods]):

    Returns:
        Response[NationalInsuranceCalculation]
    """

    return sync_detailed(
        client=client,
        tax_year=tax_year,
        gross=gross,
        ni_category=ni_category,
        pay_period=pay_period,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    ni_category: Union[Unset, None, str] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
) -> Response[NationalInsuranceCalculation]:
    """Calculate NI due.

     Calculates National Insurance Contributions due given the values specified.
    You would never need to use this API call in practice. It is provided just for information and
    testing purposes.
    Access is limited so you'll probably receive a 401 response if you try to use it.

    Args:
        tax_year (Union[Unset, None, TaxYear]):
        gross (Union[Unset, None, float]):
        ni_category (Union[Unset, None, str]):
        pay_period (Union[Unset, None, PayPeriods]):

    Returns:
        Response[NationalInsuranceCalculation]
    """

    kwargs = _get_kwargs(
        client=client,
        tax_year=tax_year,
        gross=gross,
        ni_category=ni_category,
        pay_period=pay_period,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    ni_category: Union[Unset, None, str] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
) -> Optional[NationalInsuranceCalculation]:
    """Calculate NI due.

     Calculates National Insurance Contributions due given the values specified.
    You would never need to use this API call in practice. It is provided just for information and
    testing purposes.
    Access is limited so you'll probably receive a 401 response if you try to use it.

    Args:
        tax_year (Union[Unset, None, TaxYear]):
        gross (Union[Unset, None, float]):
        ni_category (Union[Unset, None, str]):
        pay_period (Union[Unset, None, PayPeriods]):

    Returns:
        Response[NationalInsuranceCalculation]
    """

    return (
        await asyncio_detailed(
            client=client,
            tax_year=tax_year,
            gross=gross,
            ni_category=ni_category,
            pay_period=pay_period,
        )
    ).parsed



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.address import Address
from ..models.gender import Gender
from ..models.marital_status import MaritalStatus
from ..models.partner_details import PartnerDetails
from ..models.pdf_password_type import PdfPasswordType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PersonalDetails")


@attr.s(auto_attribs=True)
class PersonalDetails:
    """
    Attributes:
        marital_status (MaritalStatus):
        date_of_birth (datetime.date):
        gender (Gender):
        address (Union[Unset, Address]):
        title (Union[Unset, None, str]):
        first_name (Union[Unset, None, str]):
        middle_name (Union[Unset, None, str]):
        last_name (Union[Unset, None, str]):
        email (Union[Unset, None, str]):
        email_payslip (Union[Unset, bool]): If set to true then the employees Payslip will be sent by email when a
            PayRun is finalised.
        password_protect_payslip (Union[Unset, bool]): If set to true then when a payslip is sent by email a password
            will be set on it.
        pdf_password (Union[Unset, None, str]): Set the password to be used on PDFs. If blank then we'll create a
            password based on the PdfPasswordType property.
        pdf_password_type (Union[Unset, PdfPasswordType]):
        email_statement (Union[Unset, bool]): Only applicable to CIS Subcontractors. If set to true then we will
            automatically email a CIS Statement when a CIS300 is accepted.
        photo_url (Union[Unset, None, str]):
        telephone (Union[Unset, None, str]):
        mobile (Union[Unset, None, str]):
        state_pension_age (Union[Unset, int]): [readonly] Automatically calculated.
        ni_number (Union[Unset, None, str]):
        passport_number (Union[Unset, None, str]):
        partner_details (Union[Unset, PartnerDetails]):
    """

    marital_status: MaritalStatus
    date_of_birth: datetime.date
    gender: Gender
    address: Union[Unset, Address] = UNSET
    title: Union[Unset, None, str] = UNSET
    first_name: Union[Unset, None, str] = UNSET
    middle_name: Union[Unset, None, str] = UNSET
    last_name: Union[Unset, None, str] = UNSET
    email: Union[Unset, None, str] = UNSET
    email_payslip: Union[Unset, bool] = UNSET
    password_protect_payslip: Union[Unset, bool] = UNSET
    pdf_password: Union[Unset, None, str] = UNSET
    pdf_password_type: Union[Unset, PdfPasswordType] = UNSET
    email_statement: Union[Unset, bool] = UNSET
    photo_url: Union[Unset, None, str] = UNSET
    telephone: Union[Unset, None, str] = UNSET
    mobile: Union[Unset, None, str] = UNSET
    state_pension_age: Union[Unset, int] = UNSET
    ni_number: Union[Unset, None, str] = UNSET
    passport_number: Union[Unset, None, str] = UNSET
    partner_details: Union[Unset, PartnerDetails] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        marital_status = self.marital_status.value

        date_of_birth = self.date_of_birth.isoformat()
        gender = self.gender.value

        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        title = self.title
        first_name = self.first_name
        middle_name = self.middle_name
        last_name = self.last_name
        email = self.email
        email_payslip = self.email_payslip
        password_protect_payslip = self.password_protect_payslip
        pdf_password = self.pdf_password
        pdf_password_type: Union[Unset, str] = UNSET
        if not isinstance(self.pdf_password_type, Unset):
            pdf_password_type = self.pdf_password_type.value

        email_statement = self.email_statement
        photo_url = self.photo_url
        telephone = self.telephone
        mobile = self.mobile
        state_pension_age = self.state_pension_age
        ni_number = self.ni_number
        passport_number = self.passport_number
        partner_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.partner_details, Unset):
            partner_details = self.partner_details.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "maritalStatus": marital_status,
                "dateOfBirth": date_of_birth,
                "gender": gender,
            }
        )
        if address is not UNSET:
            field_dict["address"] = address
        if title is not UNSET:
            field_dict["title"] = title
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if middle_name is not UNSET:
            field_dict["middleName"] = middle_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if email is not UNSET:
            field_dict["email"] = email
        if email_payslip is not UNSET:
            field_dict["emailPayslip"] = email_payslip
        if password_protect_payslip is not UNSET:
            field_dict["passwordProtectPayslip"] = password_protect_payslip
        if pdf_password is not UNSET:
            field_dict["pdfPassword"] = pdf_password
        if pdf_password_type is not UNSET:
            field_dict["pdfPasswordType"] = pdf_password_type
        if email_statement is not UNSET:
            field_dict["emailStatement"] = email_statement
        if photo_url is not UNSET:
            field_dict["photoUrl"] = photo_url
        if telephone is not UNSET:
            field_dict["telephone"] = telephone
        if mobile is not UNSET:
            field_dict["mobile"] = mobile
        if state_pension_age is not UNSET:
            field_dict["statePensionAge"] = state_pension_age
        if ni_number is not UNSET:
            field_dict["niNumber"] = ni_number
        if passport_number is not UNSET:
            field_dict["passportNumber"] = passport_number
        if partner_details is not UNSET:
            field_dict["partnerDetails"] = partner_details

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        marital_status = MaritalStatus(d.pop("maritalStatus"))

        date_of_birth = isoparse(d.pop("dateOfBirth")).date()

        gender = Gender(d.pop("gender"))

        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address.from_dict(_address)

        title = d.pop("title", UNSET)

        first_name = d.pop("firstName", UNSET)

        middle_name = d.pop("middleName", UNSET)

        last_name = d.pop("lastName", UNSET)

        email = d.pop("email", UNSET)

        email_payslip = d.pop("emailPayslip", UNSET)

        password_protect_payslip = d.pop("passwordProtectPayslip", UNSET)

        pdf_password = d.pop("pdfPassword", UNSET)

        _pdf_password_type = d.pop("pdfPasswordType", UNSET)
        pdf_password_type: Union[Unset, PdfPasswordType]
        if isinstance(_pdf_password_type, Unset):
            pdf_password_type = UNSET
        else:
            pdf_password_type = PdfPasswordType(_pdf_password_type)

        email_statement = d.pop("emailStatement", UNSET)

        photo_url = d.pop("photoUrl", UNSET)

        telephone = d.pop("telephone", UNSET)

        mobile = d.pop("mobile", UNSET)

        state_pension_age = d.pop("statePensionAge", UNSET)

        ni_number = d.pop("niNumber", UNSET)

        passport_number = d.pop("passportNumber", UNSET)

        _partner_details = d.pop("partnerDetails", UNSET)
        partner_details: Union[Unset, PartnerDetails]
        if isinstance(_partner_details, Unset):
            partner_details = UNSET
        else:
            partner_details = PartnerDetails.from_dict(_partner_details)

        personal_details = cls(
            marital_status=marital_status,
            date_of_birth=date_of_birth,
            gender=gender,
            address=address,
            title=title,
            first_name=first_name,
            middle_name=middle_name,
            last_name=last_name,
            email=email,
            email_payslip=email_payslip,
            password_protect_payslip=password_protect_payslip,
            pdf_password=pdf_password,
            pdf_password_type=pdf_password_type,
            email_statement=email_statement,
            photo_url=photo_url,
            telephone=telephone,
            mobile=mobile,
            state_pension_age=state_pension_age,
            ni_number=ni_number,
            passport_number=passport_number,
            partner_details=partner_details,
        )

        return personal_details



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PayLine")


@attr.s(auto_attribs=True)
class PayLine:
    """As well as the basic pay amount for an employee there are often additions and deductions such as bonuses.
    These additions and deductions are represented using this model.

        Attributes:
            value (Union[Unset, float]): The amount to add or deduct (whether it is a deduction or addition depends on the
                PayCode used).
                If the PayCode has a CalculationType other than FixedAmount then this field will be a percentage.
                If the PayCode has a MultiplierType other than None then this field will be readonly and automatically
                calculated.
            rate (Union[Unset, None, float]): If the related  PayCode has a MultiplierType other than None then this field
                will be used as the rate per day or hour. Otherwise it isn't used
            multiplier (Union[Unset, None, float]): If the related PayCode has a MultiplierType other than None then this
                field will be used as number of days or hours
            description (Union[Unset, None, str]): A freeform description to accompany this line. It will be displayed on
                the payslip.
            attachment_order_id (Union[Unset, None, str]): [readonly] The Id of the associated AttachmentOrder. Only
                included if the Code is AEO
            pension_id (Union[Unset, None, str]): [readonly] The Id of the associated Pension. Only included if the Code is
                PENSION, PENSIONSS or PENSIONRAS
            leave_id (Union[Unset, None, str]): [readonly] The Id of the associated Leave. Only included if the PayLine is a
                result of a Leave with Statutory pay
            loan_id (Union[Unset, None, str]): [readonly] The Id of the associated Loan, if any.
            leave_statutory_days_paid (Union[Unset, None, float]): [readonly] If the PayLine is a result of a Leave with
                Statutory Pay then this property tells you how many days they've been paid for (based on their Working Pattern).
            leave_statutory_weeks_paid (Union[Unset, None, float]): [readonly] If the PayLine is a result of a Leave with
                Statutory Pay then this property tells you how many weeks they've been paid for (based on their Working
                Pattern).
            code (Union[Unset, None, str]): The Code of the PayCode this line is assigned to. The PayCode determines the
                treatment of this line when it comes to NI, Tax and Pensions as well as whether it's a deduction or addition.
            tags (Union[Unset, None, List[str]]):
            child_id (Union[Unset, str]): This is nothing but the UniqueId of the model.
            is_net_to_gross (Union[Unset, bool]): If the PayLine is a fixed ammount addition without multiplier then this
                property may be set to true so that the amount of the addition to be considered a take home pay target.
            target_net_to_gross_value (Union[Unset, None, float]): The orginal net fixed addition amount that is considered
                to be a take home pay target.
            net_to_gross_discrepancy (Union[Unset, None, float]): The discrepancy between the targeted and the calculated
                grossed up value durig a net to gross calculation.
    """

    value: Union[Unset, float] = UNSET
    rate: Union[Unset, None, float] = UNSET
    multiplier: Union[Unset, None, float] = UNSET
    description: Union[Unset, None, str] = UNSET
    attachment_order_id: Union[Unset, None, str] = UNSET
    pension_id: Union[Unset, None, str] = UNSET
    leave_id: Union[Unset, None, str] = UNSET
    loan_id: Union[Unset, None, str] = UNSET
    leave_statutory_days_paid: Union[Unset, None, float] = UNSET
    leave_statutory_weeks_paid: Union[Unset, None, float] = UNSET
    code: Union[Unset, None, str] = UNSET
    tags: Union[Unset, None, List[str]] = UNSET
    child_id: Union[Unset, str] = UNSET
    is_net_to_gross: Union[Unset, bool] = UNSET
    target_net_to_gross_value: Union[Unset, None, float] = UNSET
    net_to_gross_discrepancy: Union[Unset, None, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        value = self.value
        rate = self.rate
        multiplier = self.multiplier
        description = self.description
        attachment_order_id = self.attachment_order_id
        pension_id = self.pension_id
        leave_id = self.leave_id
        loan_id = self.loan_id
        leave_statutory_days_paid = self.leave_statutory_days_paid
        leave_statutory_weeks_paid = self.leave_statutory_weeks_paid
        code = self.code
        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        child_id = self.child_id
        is_net_to_gross = self.is_net_to_gross
        target_net_to_gross_value = self.target_net_to_gross_value
        net_to_gross_discrepancy = self.net_to_gross_discrepancy

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if value is not UNSET:
            field_dict["value"] = value
        if rate is not UNSET:
            field_dict["rate"] = rate
        if multiplier is not UNSET:
            field_dict["multiplier"] = multiplier
        if description is not UNSET:
            field_dict["description"] = description
        if attachment_order_id is not UNSET:
            field_dict["attachmentOrderId"] = attachment_order_id
        if pension_id is not UNSET:
            field_dict["pensionId"] = pension_id
        if leave_id is not UNSET:
            field_dict["leaveId"] = leave_id
        if loan_id is not UNSET:
            field_dict["loanId"] = loan_id
        if leave_statutory_days_paid is not UNSET:
            field_dict["leaveStatutoryDaysPaid"] = leave_statutory_days_paid
        if leave_statutory_weeks_paid is not UNSET:
            field_dict["leaveStatutoryWeeksPaid"] = leave_statutory_weeks_paid
        if code is not UNSET:
            field_dict["code"] = code
        if tags is not UNSET:
            field_dict["tags"] = tags
        if child_id is not UNSET:
            field_dict["childId"] = child_id
        if is_net_to_gross is not UNSET:
            field_dict["isNetToGross"] = is_net_to_gross
        if target_net_to_gross_value is not UNSET:
            field_dict["targetNetToGrossValue"] = target_net_to_gross_value
        if net_to_gross_discrepancy is not UNSET:
            field_dict["netToGrossDiscrepancy"] = net_to_gross_discrepancy

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        value = d.pop("value", UNSET)

        rate = d.pop("rate", UNSET)

        multiplier = d.pop("multiplier", UNSET)

        description = d.pop("description", UNSET)

        attachment_order_id = d.pop("attachmentOrderId", UNSET)

        pension_id = d.pop("pensionId", UNSET)

        leave_id = d.pop("leaveId", UNSET)

        loan_id = d.pop("loanId", UNSET)

        leave_statutory_days_paid = d.pop("leaveStatutoryDaysPaid", UNSET)

        leave_statutory_weeks_paid = d.pop("leaveStatutoryWeeksPaid", UNSET)

        code = d.pop("code", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        child_id = d.pop("childId", UNSET)

        is_net_to_gross = d.pop("isNetToGross", UNSET)

        target_net_to_gross_value = d.pop("targetNetToGrossValue", UNSET)

        net_to_gross_discrepancy = d.pop("netToGrossDiscrepancy", UNSET)

        pay_line = cls(
            value=value,
            rate=rate,
            multiplier=multiplier,
            description=description,
            attachment_order_id=attachment_order_id,
            pension_id=pension_id,
            leave_id=leave_id,
            loan_id=loan_id,
            leave_statutory_days_paid=leave_statutory_days_paid,
            leave_statutory_weeks_paid=leave_statutory_weeks_paid,
            code=code,
            tags=tags,
            child_id=child_id,
            is_net_to_gross=is_net_to_gross,
            target_net_to_gross_value=target_net_to_gross_value,
            net_to_gross_discrepancy=net_to_gross_discrepancy,
        )

        return pay_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EpsApprenticeshipLevy")


@attr.s(auto_attribs=True)
class EpsApprenticeshipLevy:
    """Used on an EPS to declare an Apprenticeship Levy amount

    Attributes:
        tax_month (Union[Unset, int]):
        levy_due_ytd (Union[Unset, float]):
        annual_allce (Union[Unset, float]):
    """

    tax_month: Union[Unset, int] = UNSET
    levy_due_ytd: Union[Unset, float] = UNSET
    annual_allce: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        tax_month = self.tax_month
        levy_due_ytd = self.levy_due_ytd
        annual_allce = self.annual_allce

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if tax_month is not UNSET:
            field_dict["taxMonth"] = tax_month
        if levy_due_ytd is not UNSET:
            field_dict["levyDueYTD"] = levy_due_ytd
        if annual_allce is not UNSET:
            field_dict["annualAllce"] = annual_allce

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        tax_month = d.pop("taxMonth", UNSET)

        levy_due_ytd = d.pop("levyDueYTD", UNSET)

        annual_allce = d.pop("annualAllce", UNSET)

        eps_apprenticeship_levy = cls(
            tax_month=tax_month,
            levy_due_ytd=levy_due_ytd,
            annual_allce=annual_allce,
        )

        return eps_apprenticeship_levy



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployerPayIdChanged")


@attr.s(auto_attribs=True)
class FpsEmployerPayIdChanged:
    """
    Attributes:
        payroll_id_changed_indicator (Union[Unset, None, str]):
        old_payroll_id (Union[Unset, None, str]):
    """

    payroll_id_changed_indicator: Union[Unset, None, str] = UNSET
    old_payroll_id: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        payroll_id_changed_indicator = self.payroll_id_changed_indicator
        old_payroll_id = self.old_payroll_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if payroll_id_changed_indicator is not UNSET:
            field_dict["payrollIdChangedIndicator"] = payroll_id_changed_indicator
        if old_payroll_id is not UNSET:
            field_dict["oldPayrollId"] = old_payroll_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        payroll_id_changed_indicator = d.pop("payrollIdChangedIndicator", UNSET)

        old_payroll_id = d.pop("oldPayrollId", UNSET)

        fps_employer_pay_id_changed = cls(
            payroll_id_changed_indicator=payroll_id_changed_indicator,
            old_payroll_id=old_payroll_id,
        )

        return fps_employer_pay_id_changed



from typing import Any, Dict, Optional, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.holiday_report_report_response import HolidayReportReportResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    *,
    client: Client,
    accruals: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/reports/holiday".format(
        client.base_url, employerId=employer_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(accept, Unset):
        headers["accept"] = accept

    params: Dict[str, Any] = {}
    params["accruals"] = accruals

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[HolidayReportReportResponse]:
    if response.status_code == 200:
        response_200 = HolidayReportReportResponse.from_dict(response.json())

        return response_200
    return raise_staffology_exception(response)


def _build_response(
    *, response: httpx.Response
) -> Response[HolidayReportReportResponse]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    *,
    client: Client,
    accruals: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Response[HolidayReportReportResponse]:
    """Holiday

     Returns a report summarising holiday usage for all employees.

    Args:
        employer_id (str):
        accruals (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[HolidayReportReportResponse]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        accruals=accruals,
        accept=accept,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    *,
    client: Client,
    accruals: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Optional[HolidayReportReportResponse]:
    """Holiday

     Returns a report summarising holiday usage for all employees.

    Args:
        employer_id (str):
        accruals (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[HolidayReportReportResponse]
    """

    return sync_detailed(
        employer_id=employer_id,
        client=client,
        accruals=accruals,
        accept=accept,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    *,
    client: Client,
    accruals: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Response[HolidayReportReportResponse]:
    """Holiday

     Returns a report summarising holiday usage for all employees.

    Args:
        employer_id (str):
        accruals (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[HolidayReportReportResponse]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        accruals=accruals,
        accept=accept,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    *,
    client: Client,
    accruals: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Optional[HolidayReportReportResponse]:
    """Holiday

     Returns a report summarising holiday usage for all employees.

    Args:
        employer_id (str):
        accruals (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[HolidayReportReportResponse]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            client=client,
            accruals=accruals,
            accept=accept,
        )
    ).parsed



from typing import Any, Dict, List, Optional

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.item import Item
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/account/invitations".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[Item]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Item.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[List[Item]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
) -> Response[List[Item]]:
    """Get Invitations

     Returns any pending Invitations for the authorised user.
    If the email address isn't yet verified then the Name displayed in the Item will be the users email
    address instead of the company name.
    Invitations cannot be accepted until the email address for the user is verified.

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
) -> Optional[List[Item]]:
    """Get Invitations

     Returns any pending Invitations for the authorised user.
    If the email address isn't yet verified then the Name displayed in the Item will be the users email
    address instead of the company name.
    Invitations cannot be accepted until the email address for the user is verified.

    Returns:
        Response[List[Item]]
    """

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
) -> Response[List[Item]]:
    """Get Invitations

     Returns any pending Invitations for the authorised user.
    If the email address isn't yet verified then the Name displayed in the Item will be the users email
    address instead of the company name.
    Invitations cannot be accepted until the email address for the user is verified.

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
) -> Optional[List[Item]]:
    """Get Invitations

     Returns any pending Invitations for the authorised user.
    If the email address isn't yet verified then the Name displayed in the Item will be the users email
    address instead of the company name.
    Invitations cannot be accepted until the email address for the user is verified.

    Returns:
        Response[List[Item]]
    """

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from enum import Enum


class CISTaxStatus(str, Enum):
    GROSS = "Gross"
    NETOFSTANDARDDEDUCTION = "NetOfStandardDeduction"
    NETOFHIGHERDEDUCTION = "NetOfHigherDeduction"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class PayrollValueType(str, Enum):
    BASICPAY = "BasicPay"
    GROSS = "Gross"
    GROSSFORTAX = "GrossForTax"
    GROSSFORNI = "GrossForNi"
    EMPLOYERNI = "EmployerNi"
    EMPLOYEENI = "EmployeeNi"
    EMPLOYERNIOFFPAYROLL = "EmployerNiOffPayroll"
    REALTIMECLASS1ANI = "RealTimeClass1ANi"
    TAX = "Tax"
    NETPAY = "NetPay"
    ADJUSTMENTS = "Adjustments"
    TAKEHOMEPAY = "TakeHomePay"
    NONTAXORNICPMT = "NonTaxOrNICPmt"
    ITEMSSUBJECTTOCLASS1NIC = "ItemsSubjectToClass1NIC"
    DEDNSFROMNETPAY = "DednsFromNetPay"
    TCP_TCLS = "Tcp_Tcls"
    TCP_PP = "Tcp_Pp"
    TCP_OP = "Tcp_Op"
    FLEXIDD_DEATH = "FlexiDd_Death"
    FLEXIDD_DEATH_NONTAX = "FlexiDd_Death_NonTax"
    FLEXIDD_PENSION = "FlexiDd_Pension"
    FLEXIDD_PENSION_NONTAX = "FlexiDd_Pension_NonTax"
    SMP = "Smp"
    SPP = "Spp"
    SAP = "Sap"
    SHPP = "Shpp"
    SPBP = "Spbp"
    STUDENTLOANRECOVERED = "StudentLoanRecovered"
    POSTGRADLOANRECOVERED = "PostgradLoanRecovered"
    PENSIONABLEPAY = "PensionablePay"
    NONTIERABLEPAY = "NonTierablePay"
    EMPLOYEEPENSIONCONTRIBUTION = "EmployeePensionContribution"
    EMPLOYERPENSIONCONTRIBUTION = "EmployerPensionContribution"
    EMPEEPENCONTRIBNSNOTPAID = "EmpeePenContribnsNotPaid"
    EMPEEPENCONTRIBNSPAID = "EmpeePenContribnsPaid"
    ATTACHMENTORDERDEDUCTIONS = "AttachmentOrderDeductions"
    CISDEDUCTION = "CisDeduction"
    CISVAT = "CisVat"
    CISUMBRELLAFEE = "CisUmbrellaFee"
    CISUMBRELLAFEEPOSTTAX = "CisUmbrellaFeePostTax"
    PBIK = "Pbik"
    MAPSMILES = "MapsMiles"
    UMBRELLAFEE = "UmbrellaFee"
    APPLEVYDEDUCTION = "AppLevyDeduction"
    PAYMENTAFTERLEAVING = "PaymentAfterLeaving"
    TAXONPAYMENTAFTERLEAVING = "TaxOnPaymentAfterLeaving"
    SSP = "Ssp"
    ATTACHMENTORDERADMINFEE = "AttachmentOrderAdminFee"
    EMPLOYEEPENSIONNETPAY = "EmployeePensionNetPay"
    EMPLOYEEPENSIONRAS = "EmployeePensionRas"
    EMPLOYEEPENSIONSALSAC = "EmployeePensionSalSac"
    EMPLOYEEPENSIONCONTRIBUTIONAVC = "EmployeePensionContributionAvc"
    DEDUCTIONS = "Deductions"
    ADDITIONS = "Additions"
    PENSIONABLEEARNINGS = "PensionableEarnings"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="P11NiAndStatPaymentsTotalsLine")


@attr.s(auto_attribs=True)
class P11NiAndStatPaymentsTotalsLine:
    """Summary line for the NI Contributions and Statutory Payments table in the P11 Detailed report

    Attributes:
        to_lel (Union[Unset, float]): [readonly]
        lel_to_pt (Union[Unset, float]): [readonly]
        pt_to_uel (Union[Unset, float]): [readonly]
        ees_and_ers (Union[Unset, float]): [readonly]
        ees (Union[Unset, float]): [readonly]
        class_1a (Union[Unset, float]): [readonly]
        ssp (Union[Unset, float]): [readonly]
        smp (Union[Unset, float]): [readonly]
        spp (Union[Unset, float]): [readonly]
        shpp (Union[Unset, float]): [readonly]
        sap (Union[Unset, float]): [readonly]
        spbp (Union[Unset, float]): [readonly]
    """

    to_lel: Union[Unset, float] = UNSET
    lel_to_pt: Union[Unset, float] = UNSET
    pt_to_uel: Union[Unset, float] = UNSET
    ees_and_ers: Union[Unset, float] = UNSET
    ees: Union[Unset, float] = UNSET
    class_1a: Union[Unset, float] = UNSET
    ssp: Union[Unset, float] = UNSET
    smp: Union[Unset, float] = UNSET
    spp: Union[Unset, float] = UNSET
    shpp: Union[Unset, float] = UNSET
    sap: Union[Unset, float] = UNSET
    spbp: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        to_lel = self.to_lel
        lel_to_pt = self.lel_to_pt
        pt_to_uel = self.pt_to_uel
        ees_and_ers = self.ees_and_ers
        ees = self.ees
        class_1a = self.class_1a
        ssp = self.ssp
        smp = self.smp
        spp = self.spp
        shpp = self.shpp
        sap = self.sap
        spbp = self.spbp

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if to_lel is not UNSET:
            field_dict["toLel"] = to_lel
        if lel_to_pt is not UNSET:
            field_dict["lelToPt"] = lel_to_pt
        if pt_to_uel is not UNSET:
            field_dict["ptToUel"] = pt_to_uel
        if ees_and_ers is not UNSET:
            field_dict["eesAndErs"] = ees_and_ers
        if ees is not UNSET:
            field_dict["ees"] = ees
        if class_1a is not UNSET:
            field_dict["class1A"] = class_1a
        if ssp is not UNSET:
            field_dict["ssp"] = ssp
        if smp is not UNSET:
            field_dict["smp"] = smp
        if spp is not UNSET:
            field_dict["spp"] = spp
        if shpp is not UNSET:
            field_dict["shpp"] = shpp
        if sap is not UNSET:
            field_dict["sap"] = sap
        if spbp is not UNSET:
            field_dict["spbp"] = spbp

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        to_lel = d.pop("toLel", UNSET)

        lel_to_pt = d.pop("lelToPt", UNSET)

        pt_to_uel = d.pop("ptToUel", UNSET)

        ees_and_ers = d.pop("eesAndErs", UNSET)

        ees = d.pop("ees", UNSET)

        class_1a = d.pop("class1A", UNSET)

        ssp = d.pop("ssp", UNSET)

        smp = d.pop("smp", UNSET)

        spp = d.pop("spp", UNSET)

        shpp = d.pop("shpp", UNSET)

        sap = d.pop("sap", UNSET)

        spbp = d.pop("spbp", UNSET)

        p11_ni_and_stat_payments_totals_line = cls(
            to_lel=to_lel,
            lel_to_pt=lel_to_pt,
            pt_to_uel=pt_to_uel,
            ees_and_ers=ees_and_ers,
            ees=ees,
            class_1a=class_1a,
            ssp=ssp,
            smp=smp,
            spp=spp,
            shpp=shpp,
            sap=sap,
            spbp=spbp,
        )

        return p11_ni_and_stat_payments_totals_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.teachers_pension_employment_type import TeachersPensionEmploymentType
from ..types import UNSET, Unset

T = TypeVar("T", bound="TeachersPensionDetails")


@attr.s(auto_attribs=True)
class TeachersPensionDetails:
    """Used to represent additional information needed for
    Teachers' Pensions

        Attributes:
            employment_type (Union[Unset, TeachersPensionEmploymentType]):
            full_time_salary (Union[Unset, None, int]): Up to 7 digits, in pounds. eg 24000
            part_time_salary_paid (Union[Unset, None, int]): Up to 7 digits, in pounds. eg 24000
    """

    employment_type: Union[Unset, TeachersPensionEmploymentType] = UNSET
    full_time_salary: Union[Unset, None, int] = UNSET
    part_time_salary_paid: Union[Unset, None, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employment_type: Union[Unset, str] = UNSET
        if not isinstance(self.employment_type, Unset):
            employment_type = self.employment_type.value

        full_time_salary = self.full_time_salary
        part_time_salary_paid = self.part_time_salary_paid

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employment_type is not UNSET:
            field_dict["employmentType"] = employment_type
        if full_time_salary is not UNSET:
            field_dict["fullTimeSalary"] = full_time_salary
        if part_time_salary_paid is not UNSET:
            field_dict["partTimeSalaryPaid"] = part_time_salary_paid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employment_type = d.pop("employmentType", UNSET)
        employment_type: Union[Unset, TeachersPensionEmploymentType]
        if isinstance(_employment_type, Unset):
            employment_type = UNSET
        else:
            employment_type = TeachersPensionEmploymentType(_employment_type)

        full_time_salary = d.pop("fullTimeSalary", UNSET)

        part_time_salary_paid = d.pop("partTimeSalaryPaid", UNSET)

        teachers_pension_details = cls(
            employment_type=employment_type,
            full_time_salary=full_time_salary,
            part_time_salary_paid=part_time_salary_paid,
        )

        return teachers_pension_details



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CisPartnership")


@attr.s(auto_attribs=True)
class CisPartnership:
    """If an Employee is marked as a CIS Subcontractor and is registered as a Partnership then this model provides further
    details specifically related to the CIS Partnership.

        Attributes:
            name (Union[Unset, None, str]):
            utr (Union[Unset, None, str]):
    """

    name: Union[Unset, None, str] = UNSET
    utr: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        utr = self.utr

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if utr is not UNSET:
            field_dict["utr"] = utr

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        utr = d.pop("utr", UNSET)

        cis_partnership = cls(
            name=name,
            utr=utr,
        )

        return cis_partnership



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GovTalk")


@attr.s(auto_attribs=True)
class GovTalk:
    """Part of the TaxYearConfig that our engine uses submit documents to the HMRC Gateway.
    It is used internally when our engine communicates with HMRC.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            full_payment_submission_namespace (Union[Unset, None, str]): [readonly]
            employer_payment_summary_namespace (Union[Unset, None, str]): [readonly]
            ni_no_verification_request_name_space (Union[Unset, None, str]): [readonly]
            cis_verification_request_name_space (Union[Unset, None, str]): [readonly]
            cis_300_name_space (Union[Unset, None, str]): [readonly]
            expenses_and_benefits_name_space (Union[Unset, None, str]): [readonly]
    """

    full_payment_submission_namespace: Union[Unset, None, str] = UNSET
    employer_payment_summary_namespace: Union[Unset, None, str] = UNSET
    ni_no_verification_request_name_space: Union[Unset, None, str] = UNSET
    cis_verification_request_name_space: Union[Unset, None, str] = UNSET
    cis_300_name_space: Union[Unset, None, str] = UNSET
    expenses_and_benefits_name_space: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        full_payment_submission_namespace = self.full_payment_submission_namespace
        employer_payment_summary_namespace = self.employer_payment_summary_namespace
        ni_no_verification_request_name_space = (
            self.ni_no_verification_request_name_space
        )
        cis_verification_request_name_space = self.cis_verification_request_name_space
        cis_300_name_space = self.cis_300_name_space
        expenses_and_benefits_name_space = self.expenses_and_benefits_name_space

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if full_payment_submission_namespace is not UNSET:
            field_dict[
                "fullPaymentSubmissionNamespace"
            ] = full_payment_submission_namespace
        if employer_payment_summary_namespace is not UNSET:
            field_dict[
                "employerPaymentSummaryNamespace"
            ] = employer_payment_summary_namespace
        if ni_no_verification_request_name_space is not UNSET:
            field_dict[
                "niNoVerificationRequestNameSpace"
            ] = ni_no_verification_request_name_space
        if cis_verification_request_name_space is not UNSET:
            field_dict[
                "cisVerificationRequestNameSpace"
            ] = cis_verification_request_name_space
        if cis_300_name_space is not UNSET:
            field_dict["cis300NameSpace"] = cis_300_name_space
        if expenses_and_benefits_name_space is not UNSET:
            field_dict[
                "expensesAndBenefitsNameSpace"
            ] = expenses_and_benefits_name_space

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        full_payment_submission_namespace = d.pop(
            "fullPaymentSubmissionNamespace", UNSET
        )

        employer_payment_summary_namespace = d.pop(
            "employerPaymentSummaryNamespace", UNSET
        )

        ni_no_verification_request_name_space = d.pop(
            "niNoVerificationRequestNameSpace", UNSET
        )

        cis_verification_request_name_space = d.pop(
            "cisVerificationRequestNameSpace", UNSET
        )

        cis_300_name_space = d.pop("cis300NameSpace", UNSET)

        expenses_and_benefits_name_space = d.pop("expensesAndBenefitsNameSpace", UNSET)

        gov_talk = cls(
            full_payment_submission_namespace=full_payment_submission_namespace,
            employer_payment_summary_namespace=employer_payment_summary_namespace,
            ni_no_verification_request_name_space=ni_no_verification_request_name_space,
            cis_verification_request_name_space=cis_verification_request_name_space,
            cis_300_name_space=cis_300_name_space,
            expenses_and_benefits_name_space=expenses_and_benefits_name_space,
        )

        return gov_talk



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.p11_line import P11Line
from ..models.report import Report
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11")


@attr.s(auto_attribs=True)
class P11:
    """A P11 Report summarises payments and deductions made to an employee.
    Our Reports API can return this to you in many formats including as a PDF file
    If you request is as a JSOn object then it is represented using this model.

        Attributes:
            employer_name (Union[Unset, None, str]): [readonly]
            payroll_code (Union[Unset, None, str]): [readonly]
            employer_office_no (Union[Unset, None, str]): [readonly]
            employer_paye_ref (Union[Unset, None, str]): [readonly]
            firstname (Union[Unset, None, str]): [readonly]
            surname (Union[Unset, None, str]): [readonly]
            ni_number (Union[Unset, None, str]): [readonly]
            tax_code (Union[Unset, None, str]): [readonly]
            date_of_birth (Union[Unset, datetime.date]): [readonly]
            join_date (Union[Unset, datetime.date]): [readonly]
            leave_date (Union[Unset, None, datetime.date]): [readonly]
            lines (Union[Unset, None, List[P11Line]]): [readonly]
            report (Union[Unset, Report]):
            tax_year (Union[Unset, TaxYear]):
    """

    employer_name: Union[Unset, None, str] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    employer_office_no: Union[Unset, None, str] = UNSET
    employer_paye_ref: Union[Unset, None, str] = UNSET
    firstname: Union[Unset, None, str] = UNSET
    surname: Union[Unset, None, str] = UNSET
    ni_number: Union[Unset, None, str] = UNSET
    tax_code: Union[Unset, None, str] = UNSET
    date_of_birth: Union[Unset, datetime.date] = UNSET
    join_date: Union[Unset, datetime.date] = UNSET
    leave_date: Union[Unset, None, datetime.date] = UNSET
    lines: Union[Unset, None, List[P11Line]] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employer_name = self.employer_name
        payroll_code = self.payroll_code
        employer_office_no = self.employer_office_no
        employer_paye_ref = self.employer_paye_ref
        firstname = self.firstname
        surname = self.surname
        ni_number = self.ni_number
        tax_code = self.tax_code
        date_of_birth: Union[Unset, str] = UNSET
        if not isinstance(self.date_of_birth, Unset):
            date_of_birth = self.date_of_birth.isoformat()

        join_date: Union[Unset, str] = UNSET
        if not isinstance(self.join_date, Unset):
            join_date = self.join_date.isoformat()

        leave_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.leave_date, Unset):
            leave_date = self.leave_date.isoformat() if self.leave_date else None

        lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.lines, Unset):
            if self.lines is None:
                lines = None
            else:
                lines = []
                for lines_item_data in self.lines:
                    lines_item = lines_item_data.to_dict()

                    lines.append(lines_item)

        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employer_name is not UNSET:
            field_dict["employerName"] = employer_name
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if employer_office_no is not UNSET:
            field_dict["employerOfficeNo"] = employer_office_no
        if employer_paye_ref is not UNSET:
            field_dict["employerPayeRef"] = employer_paye_ref
        if firstname is not UNSET:
            field_dict["firstname"] = firstname
        if surname is not UNSET:
            field_dict["surname"] = surname
        if ni_number is not UNSET:
            field_dict["niNumber"] = ni_number
        if tax_code is not UNSET:
            field_dict["taxCode"] = tax_code
        if date_of_birth is not UNSET:
            field_dict["dateOfBirth"] = date_of_birth
        if join_date is not UNSET:
            field_dict["joinDate"] = join_date
        if leave_date is not UNSET:
            field_dict["leaveDate"] = leave_date
        if lines is not UNSET:
            field_dict["lines"] = lines
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employer_name = d.pop("employerName", UNSET)

        payroll_code = d.pop("payrollCode", UNSET)

        employer_office_no = d.pop("employerOfficeNo", UNSET)

        employer_paye_ref = d.pop("employerPayeRef", UNSET)

        firstname = d.pop("firstname", UNSET)

        surname = d.pop("surname", UNSET)

        ni_number = d.pop("niNumber", UNSET)

        tax_code = d.pop("taxCode", UNSET)

        _date_of_birth = d.pop("dateOfBirth", UNSET)
        date_of_birth: Union[Unset, datetime.date]
        if isinstance(_date_of_birth, Unset):
            date_of_birth = UNSET
        else:
            date_of_birth = isoparse(_date_of_birth).date()

        _join_date = d.pop("joinDate", UNSET)
        join_date: Union[Unset, datetime.date]
        if isinstance(_join_date, Unset):
            join_date = UNSET
        else:
            join_date = isoparse(_join_date).date()

        _leave_date = d.pop("leaveDate", UNSET)
        leave_date: Union[Unset, None, datetime.date]
        if _leave_date is None:
            leave_date = None
        elif isinstance(_leave_date, Unset):
            leave_date = UNSET
        else:
            leave_date = isoparse(_leave_date).date()

        lines = []
        _lines = d.pop("lines", UNSET)
        for lines_item_data in _lines or []:
            lines_item = P11Line.from_dict(lines_item_data)

            lines.append(lines_item)

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        p11 = cls(
            employer_name=employer_name,
            payroll_code=payroll_code,
            employer_office_no=employer_office_no,
            employer_paye_ref=employer_paye_ref,
            firstname=firstname,
            surname=surname,
            ni_number=ni_number,
            tax_code=tax_code,
            date_of_birth=date_of_birth,
            join_date=join_date,
            leave_date=leave_date,
            lines=lines,
            report=report,
            tax_year=tax_year,
        )

        return p11



from enum import Enum


class AverageWeeklyEarningsResult(str, Enum):
    SUCCESS = "Success"
    EARNINGSBELOWTHRESHOLD = "EarningsBelowThreshold"
    NOTENOUGHPAYROLLDATA = "NotEnoughPayrollData"
    TAXYEARNOTSUPPORTED = "TaxYearNotSupported"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployeeContactPostalAddress")


@attr.s(auto_attribs=True)
class PapdisEmployeeContactPostalAddress:
    """
    Attributes:
        address1 (Union[Unset, None, str]): [readonly]
        address2 (Union[Unset, None, str]): [readonly]
        address3 (Union[Unset, None, str]): [readonly]
        address4 (Union[Unset, None, str]): [readonly]
        postcode (Union[Unset, None, str]): [readonly]
        country (Union[Unset, None, str]): [readonly]
    """

    address1: Union[Unset, None, str] = UNSET
    address2: Union[Unset, None, str] = UNSET
    address3: Union[Unset, None, str] = UNSET
    address4: Union[Unset, None, str] = UNSET
    postcode: Union[Unset, None, str] = UNSET
    country: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        address1 = self.address1
        address2 = self.address2
        address3 = self.address3
        address4 = self.address4
        postcode = self.postcode
        country = self.country

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if address1 is not UNSET:
            field_dict["address1"] = address1
        if address2 is not UNSET:
            field_dict["address2"] = address2
        if address3 is not UNSET:
            field_dict["address3"] = address3
        if address4 is not UNSET:
            field_dict["address4"] = address4
        if postcode is not UNSET:
            field_dict["postcode"] = postcode
        if country is not UNSET:
            field_dict["country"] = country

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        address1 = d.pop("address1", UNSET)

        address2 = d.pop("address2", UNSET)

        address3 = d.pop("address3", UNSET)

        address4 = d.pop("address4", UNSET)

        postcode = d.pop("postcode", UNSET)

        country = d.pop("country", UNSET)

        papdis_employee_contact_postal_address = cls(
            address1=address1,
            address2=address2,
            address3=address3,
            address4=address4,
            postcode=postcode,
            country=country,
        )

        return papdis_employee_contact_postal_address



from enum import Enum


class ReportCustomCssOption(str, Enum):
    USEDEFAULT = "UseDefault"
    APPENDTODEFAULT = "AppendToDefault"
    REPLACEDEFAULT = "ReplaceDefault"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UmbrellaSettings")


@attr.s(auto_attribs=True)
class UmbrellaSettings:
    """
    Attributes:
        enabled (Union[Unset, bool]):
        charge_per_timesheet (Union[Unset, float]):
        apprenticeship_levy_dedn_rate (Union[Unset, float]):
        holiday_rate (Union[Unset, float]):
        dpsb_code (Union[Unset, None, str]):
        expenses_code (Union[Unset, None, str]):
        gross_deduction_code (Union[Unset, None, str]):
        holiday_code (Union[Unset, None, str]):
        cis_fee_code (Union[Unset, None, str]):
        detail_fee_in_comment (Union[Unset, bool]):
    """

    enabled: Union[Unset, bool] = UNSET
    charge_per_timesheet: Union[Unset, float] = UNSET
    apprenticeship_levy_dedn_rate: Union[Unset, float] = UNSET
    holiday_rate: Union[Unset, float] = UNSET
    dpsb_code: Union[Unset, None, str] = UNSET
    expenses_code: Union[Unset, None, str] = UNSET
    gross_deduction_code: Union[Unset, None, str] = UNSET
    holiday_code: Union[Unset, None, str] = UNSET
    cis_fee_code: Union[Unset, None, str] = UNSET
    detail_fee_in_comment: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        charge_per_timesheet = self.charge_per_timesheet
        apprenticeship_levy_dedn_rate = self.apprenticeship_levy_dedn_rate
        holiday_rate = self.holiday_rate
        dpsb_code = self.dpsb_code
        expenses_code = self.expenses_code
        gross_deduction_code = self.gross_deduction_code
        holiday_code = self.holiday_code
        cis_fee_code = self.cis_fee_code
        detail_fee_in_comment = self.detail_fee_in_comment

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if charge_per_timesheet is not UNSET:
            field_dict["chargePerTimesheet"] = charge_per_timesheet
        if apprenticeship_levy_dedn_rate is not UNSET:
            field_dict["apprenticeshipLevyDednRate"] = apprenticeship_levy_dedn_rate
        if holiday_rate is not UNSET:
            field_dict["holidayRate"] = holiday_rate
        if dpsb_code is not UNSET:
            field_dict["dpsbCode"] = dpsb_code
        if expenses_code is not UNSET:
            field_dict["expensesCode"] = expenses_code
        if gross_deduction_code is not UNSET:
            field_dict["grossDeductionCode"] = gross_deduction_code
        if holiday_code is not UNSET:
            field_dict["holidayCode"] = holiday_code
        if cis_fee_code is not UNSET:
            field_dict["cisFeeCode"] = cis_fee_code
        if detail_fee_in_comment is not UNSET:
            field_dict["detailFeeInComment"] = detail_fee_in_comment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled", UNSET)

        charge_per_timesheet = d.pop("chargePerTimesheet", UNSET)

        apprenticeship_levy_dedn_rate = d.pop("apprenticeshipLevyDednRate", UNSET)

        holiday_rate = d.pop("holidayRate", UNSET)

        dpsb_code = d.pop("dpsbCode", UNSET)

        expenses_code = d.pop("expensesCode", UNSET)

        gross_deduction_code = d.pop("grossDeductionCode", UNSET)

        holiday_code = d.pop("holidayCode", UNSET)

        cis_fee_code = d.pop("cisFeeCode", UNSET)

        detail_fee_in_comment = d.pop("detailFeeInComment", UNSET)

        umbrella_settings = cls(
            enabled=enabled,
            charge_per_timesheet=charge_per_timesheet,
            apprenticeship_levy_dedn_rate=apprenticeship_levy_dedn_rate,
            holiday_rate=holiday_rate,
            dpsb_code=dpsb_code,
            expenses_code=expenses_code,
            gross_deduction_code=gross_deduction_code,
            holiday_code=holiday_code,
            cis_fee_code=cis_fee_code,
            detail_fee_in_comment=detail_fee_in_comment,
        )

        return umbrella_settings



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.background_task_status import BackgroundTaskStatus
from ..models.email_attachment import EmailAttachment
from ..types import UNSET, Unset

T = TypeVar("T", bound="TenantEmail")


@attr.s(auto_attribs=True)
class TenantEmail:
    """
    Attributes:
        to_name (str):
        to_email (str):
        subject (str):
        status_message (Union[Unset, None, str]):
        button_text (Union[Unset, None, str]):
        button_link (Union[Unset, None, str]):
        body_preview (Union[Unset, None, str]):
        body (Union[Unset, None, str]):
        after_btn_body (Union[Unset, None, str]):
        status (Union[Unset, BackgroundTaskStatus]):
        status_date (Union[Unset, datetime.datetime]):
        status_message (Union[Unset, None, str]):
        send_attempts (Union[Unset, int]):
        web_app_base_url (Union[Unset, None, str]):
        attachments (Union[Unset, None, List[EmailAttachment]]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    to_name: str
    to_email: str
    subject: str
    status_message: Union[Unset, None, str] = UNSET
    button_text: Union[Unset, None, str] = UNSET
    button_link: Union[Unset, None, str] = UNSET
    body_preview: Union[Unset, None, str] = UNSET
    body: Union[Unset, None, str] = UNSET
    after_btn_body: Union[Unset, None, str] = UNSET
    status: Union[Unset, BackgroundTaskStatus] = UNSET
    status_date: Union[Unset, datetime.datetime] = UNSET
    send_attempts: Union[Unset, int] = UNSET
    web_app_base_url: Union[Unset, None, str] = UNSET
    attachments: Union[Unset, None, List[EmailAttachment]] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        to_name = self.to_name
        to_email = self.to_email
        subject = self.subject
        status_message = self.status_message
        button_text = self.button_text
        button_link = self.button_link
        body_preview = self.body_preview
        body = self.body
        after_btn_body = self.after_btn_body
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        status_date: Union[Unset, str] = UNSET
        if not isinstance(self.status_date, Unset):
            status_date = self.status_date.isoformat()

        status_message = self.status_message
        send_attempts = self.send_attempts
        web_app_base_url = self.web_app_base_url
        attachments: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.attachments, Unset):
            if self.attachments is None:
                attachments = None
            else:
                attachments = []
                for attachments_item_data in self.attachments:
                    attachments_item = attachments_item_data.to_dict()

                    attachments.append(attachments_item)

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "toName": to_name,
                "toEmail": to_email,
                "subject": subject,
            }
        )
        if status_message is not UNSET:
            field_dict["_statusMessage"] = status_message
        if button_text is not UNSET:
            field_dict["buttonText"] = button_text
        if button_link is not UNSET:
            field_dict["buttonLink"] = button_link
        if body_preview is not UNSET:
            field_dict["bodyPreview"] = body_preview
        if body is not UNSET:
            field_dict["body"] = body
        if after_btn_body is not UNSET:
            field_dict["afterBtnBody"] = after_btn_body
        if status is not UNSET:
            field_dict["status"] = status
        if status_date is not UNSET:
            field_dict["statusDate"] = status_date
        if status_message is not UNSET:
            field_dict["statusMessage"] = status_message
        if send_attempts is not UNSET:
            field_dict["sendAttempts"] = send_attempts
        if web_app_base_url is not UNSET:
            field_dict["webAppBaseUrl"] = web_app_base_url
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        to_name = d.pop("toName")

        to_email = d.pop("toEmail")

        subject = d.pop("subject")

        status_message = d.pop("_statusMessage", UNSET)

        button_text = d.pop("buttonText", UNSET)

        button_link = d.pop("buttonLink", UNSET)

        body_preview = d.pop("bodyPreview", UNSET)

        body = d.pop("body", UNSET)

        after_btn_body = d.pop("afterBtnBody", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, BackgroundTaskStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = BackgroundTaskStatus(_status)

        _status_date = d.pop("statusDate", UNSET)
        status_date: Union[Unset, datetime.datetime]
        if isinstance(_status_date, Unset):
            status_date = UNSET
        else:
            status_date = isoparse(_status_date)

        status_message = d.pop("statusMessage", UNSET)

        send_attempts = d.pop("sendAttempts", UNSET)

        web_app_base_url = d.pop("webAppBaseUrl", UNSET)

        attachments = []
        _attachments = d.pop("attachments", UNSET)
        for attachments_item_data in _attachments or []:
            attachments_item = EmailAttachment.from_dict(attachments_item_data)

            attachments.append(attachments_item)

        id = d.pop("id", UNSET)

        tenant_email = cls(
            to_name=to_name,
            to_email=to_email,
            subject=subject,
            status_message=status_message,
            button_text=button_text,
            button_link=button_link,
            body_preview=body_preview,
            body=body,
            after_btn_body=after_btn_body,
            status=status,
            status_date=status_date,
            send_attempts=send_attempts,
            web_app_base_url=web_app_base_url,
            attachments=attachments,
            id=id,
        )

        return tenant_email
